# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.functional.shared_file_system import base


class AvailabilityZoneTest(base.BaseSharedFileSystemTest):
    min_microversion = '2.7'

    def test_availability_zones(self):
        azs = self.user_cloud.shared_file_system.availability_zones()
        self.assertGreater(len(list(azs)), 0)
        for az in azs:
            for attribute in ('id', 'name', 'created_at', 'updated_at'):
                self.assertTrue(hasattr(az, attribute))
                self.assertIsInstance(getattr(az, attribute), 'str')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc
import typing as ty
import warnings

from requests import structures

from openstack import format
from openstack import warnings as os_warnings

_SEEN_FORMAT = '{name}_seen'

_T1 = ty.TypeVar('_T1')
_T2 = ty.TypeVar('_T2')
_T3 = ty.TypeVar('_T3', str, bool, int, float)


# case 1: data_type is unset -> return value as-is
@ty.overload
def _convert_type(
    value: _T1,
    data_type: None,
    list_type: None = None,
) -> _T1: ...


# case 2: data_type is primitive type -> return value as said primitive type
@ty.overload
def _convert_type(
    value: _T1,
    data_type: type[_T3],
    list_type: None = None,
) -> _T3: ...


# case 3: data_type is list, no list_type -> return value as list of whatever
# we got
@ty.overload
def _convert_type(
    value: _T1,
    data_type: type[list[ty.Any]],
    list_type: None = None,
) -> list[_T1]: ...


# case 4: data_type is list, list_type is primitive type -> return value as
# list of said primitive type
@ty.overload
def _convert_type(
    value: ty.Any,
    data_type: type[list[ty.Any]],
    list_type: type[_T3],
) -> list[_T3]: ...


# case 5: data_type is dict or Resource -> return value as dict/Resource
@ty.overload
def _convert_type(
    value: ty.Any,
    data_type: type[dict[ty.Any, ty.Any]],
    list_type: None = None,
) -> dict[ty.Any, ty.Any]: ...


# case 6: data_type is a Formatter -> return value after conversion
@ty.overload
def _convert_type(
    value: ty.Any,
    data_type: type[format.Formatter[type[_T2]]],
    list_type: None = None,
) -> _T2: ...


def _convert_type(
    value: _T1,
    data_type: type[
        _T3 | list[ty.Any] | dict[ty.Any, ty.Any] | format.Formatter[_T2],
    ]
    | None,
    list_type: type[_T3] | None = None,
) -> _T1 | _T3 | list[_T3] | list[_T1] | dict[ty.Any, ty.Any] | _T2:
    # This should allow handling list of dicts that have their own
    # Component type directly. See openstack/compute/v2/limits.py
    # and the RateLimit type for an example.
    if data_type is None:
        return value
    elif issubclass(data_type, list):
        if isinstance(value, list | set | tuple):
            if not list_type:
                return data_type(value)
            return [_convert_type(x, list_type) for x in value]
        elif list_type:
            return [_convert_type(value, list_type)]
        else:
            return [value]
    elif isinstance(value, data_type):
        return value
    elif issubclass(data_type, dict):
        if isinstance(value, dict):
            return data_type(**value)
        # TODO(stephenfin): This should be a warning/error
        return dict()
    elif issubclass(data_type, format.Formatter):
        return data_type.deserialize(value)
    elif issubclass(data_type, bool):
        return data_type(value)
    elif issubclass(data_type, int | float):
        if isinstance(value, int | float):
            return data_type(value)
        if isinstance(value, str):
            if issubclass(data_type, int) and value.isdigit():
                return data_type(value)
            elif issubclass(data_type, float) and (
                x.isdigit() for x in value.split()
            ):
                return data_type(value)
        return data_type()

    # at this point we expect to have a str and you can convert basically
    # anything to a string, but there could be untyped code out there passing
    # random monstrosities so we need the try-catch to be safe
    try:
        return data_type(value)
    except ValueError:
        return data_type()


class _BaseComponent(abc.ABC):
    # The name this component is being tracked as in the Resource
    key: ty.ClassVar[str]
    # The class to be used for mappings
    _map_cls: ty.ClassVar[type[ty.MutableMapping[str, ty.Any]]] = dict

    name: str
    data_type: ty.Any | None
    default: ty.Any
    alias: str | None
    aka: str | None
    alternate_id: bool
    list_type: ty.Any | None
    coerce_to_default: bool
    deprecated: bool
    deprecation_reason: str | None

    def __init__(
        self,
        name: str,
        type: ty.Any | None = None,
        default: ty.Any = None,
        alias: str | None = None,
        aka: str | None = None,
        alternate_id: bool = False,
        list_type: ty.Any | None = None,
        coerce_to_default: bool = False,
        deprecated: bool = False,
        deprecation_reason: str | None = None,
    ):
        """A typed descriptor for a component that makes up a Resource

        :param name: The name this component exists as on the server
        :param type:
            The type this component is expected to be by the server.
            By default this is None, meaning any value you specify
            will work. If you specify type=dict and then set a
            component to a string, __set__ will fail, for example.
        :param default: Typically None, but any other default can be set.
        :param alias: If set, alternative attribute on object to return.
        :param aka: If set, additional name attribute would be available under.
        :param alternate_id: When `True`, this property is known internally as
            a value that can be sent with requests that require an ID but when
            `id` is not a name the Resource has. This is a relatively uncommon
            case, and this setting should only be used once per Resource.
        :param list_type: If type is `list`, list_type designates what the type
            of the elements of the list should be.
        :param coerce_to_default: If the Component is None or not present,
            force the given default to be used. If a default is not given but a
            type is given, construct an empty version of the type in question.
        :param deprecated: Indicates if the option is deprecated. If it is, we
            display a warning message to the user.
        :param deprecation_reason: Custom deprecation message.
        """
        self.name = name
        self.data_type = type
        if type is not None and coerce_to_default and not default:
            self.default = type()
        else:
            self.default = default
        self.alias = alias
        self.aka = aka
        self.alternate_id = alternate_id
        self.list_type = list_type
        self.coerce_to_default = coerce_to_default

        self.deprecated = deprecated
        self.deprecation_reason = deprecation_reason

    def __get__(
        self,
        instance: object,
        owner: type[object] | None = None,
    ) -> ty.Any:
        if instance is None:
            return self

        attributes = getattr(instance, self.key)

        try:
            value = attributes[self.name]
        except KeyError:
            value = self.default
            if self.alias:
                # Resource attributes can be aliased to each other. If neither
                # of them exist, then simply doing a
                # getattr(instance, self.alias) here sends things into
                # infinite recursion (this _get method is what gets called
                # when getattr(instance) is called.
                # To combat that, we set a flag on the instance saying that
                # we have seen the current name, and we check before trying
                # to resolve the alias if there is already a flag set for that
                # alias name. We then remove the seen flag for ourselves after
                # we exit the alias getattr to clean up after ourselves for
                # the next time.
                alias_flag = _SEEN_FORMAT.format(name=self.alias)
                if not getattr(instance, alias_flag, False):
                    seen_flag = _SEEN_FORMAT.format(name=self.name)
                    # Prevent infinite recursion
                    setattr(instance, seen_flag, True)
                    value = getattr(instance, self.alias)
                    delattr(instance, seen_flag)
            self.warn_if_deprecated_property(value)
            return value

        # self.data_type() should not be called on None objects.
        if value is None:
            return None

        # This warning are pretty intrusive. Every time attribute is accessed
        # a warning is being thrown. In neutron clients we have way too many
        # places that still refer to tenant_id even though they may also
        # properly support project_id. For now we silence tenant_id warnings.
        if self.name != "tenant_id":
            self.warn_if_deprecated_property(value)

        return _convert_type(value, self.data_type, self.list_type)

    @property
    def type(self) -> ty.Any | None:
        # deprecated alias proxy
        return self.data_type

    def warn_if_deprecated_property(self, value: ty.Any) -> None:
        deprecated = object.__getattribute__(self, 'deprecated')
        deprecation_reason = object.__getattribute__(
            self,
            'deprecation_reason',
        )

        if value and deprecated:
            warnings.warn(
                "The field {!r} has been deprecated. {}".format(
                    self.name, deprecation_reason or "Avoid usage."
                ),
                os_warnings.RemovedFieldWarning,
            )

    def __set__(self, instance: object, value: ty.Any) -> None:
        if self.coerce_to_default and value is None:
            value_ = self.default
        elif value != self.default:
            value_ = _convert_type(value, self.data_type, self.list_type)
        else:
            value_ = value

        attributes = getattr(instance, self.key)
        attributes[self.name] = value_

    def __delete__(self, instance: object) -> None:
        try:
            attributes = getattr(instance, self.key)
            del attributes[self.name]
        except KeyError:
            pass


class Body(_BaseComponent):
    """Body attributes"""

    key = "_body"


class Header(_BaseComponent):
    """Header attributes"""

    key = "_header"
    _map_cls = structures.CaseInsensitiveDict


class URI(_BaseComponent):
    """URI attributes"""

    key = "_uri"


class Computed(_BaseComponent):
    """Computed attributes"""

    key = "_computed"



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
from unittest import mock

from keystoneauth1 import adapter

from openstack.block_storage.v2 import volume
from openstack.tests.unit import base

FAKE_ID = "6685584b-1eac-4da6-b5c3-555430cf68ff"
IMAGE_METADATA = {
    'container_format': 'bare',
    'min_ram': '64',
    'disk_format': 'qcow2',
    'image_name': 'TestVM',
    'image_id': '625d4f2c-cf67-4af3-afb6-c7220f766947',
    'checksum': '64d7c1cd2b6f60c92c14662941cb7913',
    'min_disk': '0',
    'size': '13167616',
}

VOLUME = {
    "status": "creating",
    "name": "my_volume",
    "attachments": [],
    "availability_zone": "nova",
    "bootable": "false",
    "created_at": "2015-03-09T12:14:57.233772",
    "updated_at": None,
    "description": "something",
    "volume_type": "some_type",
    "snapshot_id": "93c2e2aa-7744-4fd6-a31a-80c4726b08d7",
    "source_volid": None,
    "imageRef": "some_image",
    "metadata": {},
    "volume_image_metadata": IMAGE_METADATA,
    "id": FAKE_ID,
    "size": 10,
    "os-vol-host-attr:host": "127.0.0.1",
    "os-vol-tenant-attr:tenant_id": "some tenant",
    "os-vol-mig-status-attr:migstat": "done",
    "os-vol-mig-status-attr:name_id": "93c2e2aa-7744-4fd6-a31a-80c4726b08d7",
    "replication_status": "nah",
    "os-volume-replication:extended_status": "really nah",
    "consistencygroup_id": "123asf-asdf123",
    "os-volume-replication:driver_data": "ahasadfasdfasdfasdfsdf",
    "encrypted": "false",
    "OS-SCH-HNT:scheduler_hints": {
        "same_host": [
            "a0cf03a5-d921-4877-bb5c-86d26cf818e1",
            "8c19174f-4220-44f0-824a-cd1eeef10287",
        ]
    },
}


class TestVolume(base.TestCase):
    def test_basic(self):
        sot = volume.Volume(VOLUME)
        self.assertEqual("volume", sot.resource_key)
        self.assertEqual("volumes", sot.resources_key)
        self.assertEqual("/volumes", sot.base_path)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

        self.assertDictEqual(
            {
                "name": "name",
                "status": "status",
                "all_projects": "all_tenants",
                "project_id": "project_id",
                "limit": "limit",
                "marker": "marker",
            },
            sot._query_mapping._mapping,
        )

    def test_create(self):
        sot = volume.Volume(**VOLUME)
        self.assertEqual(VOLUME["id"], sot.id)
        self.assertEqual(VOLUME["status"], sot.status)
        self.assertEqual(VOLUME["attachments"], sot.attachments)
        self.assertEqual(VOLUME["availability_zone"], sot.availability_zone)
        self.assertFalse(sot.is_bootable)
        self.assertEqual(VOLUME["created_at"], sot.created_at)
        self.assertEqual(VOLUME["updated_at"], sot.updated_at)
        self.assertEqual(VOLUME["description"], sot.description)
        self.assertEqual(VOLUME["volume_type"], sot.volume_type)
        self.assertEqual(VOLUME["snapshot_id"], sot.snapshot_id)
        self.assertEqual(VOLUME["source_volid"], sot.source_volume_id)
        self.assertEqual(VOLUME["metadata"], sot.metadata)
        self.assertEqual(
            VOLUME["volume_image_metadata"], sot.volume_image_metadata
        )
        self.assertEqual(VOLUME["size"], sot.size)
        self.assertEqual(VOLUME["imageRef"], sot.image_id)
        self.assertEqual(VOLUME["os-vol-host-attr:host"], sot.host)
        self.assertEqual(
            VOLUME["os-vol-tenant-attr:tenant_id"], sot.project_id
        )
        self.assertEqual(
            VOLUME["os-vol-mig-status-attr:migstat"], sot.migration_status
        )
        self.assertEqual(
            VOLUME["os-vol-mig-status-attr:name_id"], sot.migration_id
        )
        self.assertEqual(VOLUME["replication_status"], sot.replication_status)
        self.assertEqual(
            VOLUME["os-volume-replication:extended_status"],
            sot.extended_replication_status,
        )
        self.assertEqual(
            VOLUME["consistencygroup_id"], sot.consistency_group_id
        )
        self.assertEqual(
            VOLUME["os-volume-replication:driver_data"],
            sot.replication_driver_data,
        )
        self.assertDictEqual(
            VOLUME["OS-SCH-HNT:scheduler_hints"], sot.scheduler_hints
        )
        self.assertFalse(sot.is_encrypted)


class TestVolumeActions(TestVolume):
    def setUp(self):
        super().setUp()
        self.resp = mock.Mock()
        self.resp.body = None
        self.resp.status_code = 200
        self.resp.json = mock.Mock(return_value=self.resp.body)
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock(return_value=self.resp)
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

    def test_extend(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.extend(self.sess, '20'))

        url = f'volumes/{FAKE_ID}/action'
        body = {"os-extend": {"new_size": "20"}}
        self.sess.post.assert_called_with(url, json=body)

    def test_set_volume_readonly(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.set_readonly(self.sess, True))

        url = f'volumes/{FAKE_ID}/action'
        body = {'os-update_readonly_flag': {'readonly': True}}
        self.sess.post.assert_called_with(url, json=body)

    def test_set_volume_readonly_false(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.set_readonly(self.sess, False))

        url = f'volumes/{FAKE_ID}/action'
        body = {'os-update_readonly_flag': {'readonly': False}}
        self.sess.post.assert_called_with(url, json=body)

    def test_set_volume_bootable(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.set_bootable_status(self.sess))

        url = f'volumes/{FAKE_ID}/action'
        body = {'os-set_bootable': {'bootable': True}}
        self.sess.post.assert_called_with(url, json=body)

    def test_set_volume_bootable_false(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.set_bootable_status(self.sess, False))

        url = f'volumes/{FAKE_ID}/action'
        body = {'os-set_bootable': {'bootable': False}}
        self.sess.post.assert_called_with(url, json=body)

    def test_set_image_metadata(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.set_image_metadata(self.sess, {'foo': 'bar'}))

        url = f'volumes/{FAKE_ID}/action'
        body = {'os-set_image_metadata': {'metadata': {'foo': 'bar'}}}
        self.sess.post.assert_called_with(url, json=body)

    def test_delete_image_metadata(self):
        _volume = copy.deepcopy(VOLUME)
        _volume['metadata'] = {
            'foo': 'bar',
            'baz': 'wow',
        }
        sot = volume.Volume(**_volume)

        self.assertIsNone(sot.delete_image_metadata(self.sess))

        url = f'volumes/{FAKE_ID}/action'
        body_a = {'os-unset_image_metadata': 'foo'}
        body_b = {'os-unset_image_metadata': 'baz'}
        self.sess.post.assert_has_calls(
            [
                mock.call(url, json=body_a),
                mock.call(url, json=body_b),
            ]
        )

    def test_delete_image_metadata_item(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.delete_image_metadata_item(self.sess, 'foo'))

        url = f'volumes/{FAKE_ID}/action'
        body = {'os-unset_image_metadata': 'foo'}
        self.sess.post.assert_called_with(url, json=body)

    def test_reset_status(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.reset_status(self.sess, '1', '2', '3'))

        url = f'volumes/{FAKE_ID}/action'
        body = {
            'os-reset_status': {
                'status': '1',
                'attach_status': '2',
                'migration_status': '3',
            }
        }
        self.sess.post.assert_called_with(url, json=body)

    def test_reset_status__single_option(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.reset_status(self.sess, status='1'))

        url = f'volumes/{FAKE_ID}/action'
        body = {
            'os-reset_status': {
                'status': '1',
            }
        }
        self.sess.post.assert_called_with(url, json=body)

    def test_attach_instance(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.attach(self.sess, '1', '2'))

        url = f'volumes/{FAKE_ID}/action'
        body = {'os-attach': {'mountpoint': '1', 'instance_uuid': '2'}}
        self.sess.post.assert_called_with(url, json=body)

    def test_detach(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.detach(self.sess, '1'))

        url = f'volumes/{FAKE_ID}/action'
        body = {'os-detach': {'attachment_id': '1'}}
        self.sess.post.assert_called_with(url, json=body)

    def test_detach_force(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.detach(self.sess, '1', force=True))

        url = f'volumes/{FAKE_ID}/action'
        body = {'os-force_detach': {'attachment_id': '1'}}
        self.sess.post.assert_called_with(url, json=body)

    def test_unmanage(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.unmanage(self.sess))

        url = f'volumes/{FAKE_ID}/action'
        body = {'os-unmanage': None}
        self.sess.post.assert_called_with(url, json=body)

    def test_retype(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.retype(self.sess, '1'))

        url = f'volumes/{FAKE_ID}/action'
        body = {'os-retype': {'new_type': '1'}}
        self.sess.post.assert_called_with(url, json=body)

    def test_retype_mp(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.retype(self.sess, '1', migration_policy='2'))

        url = f'volumes/{FAKE_ID}/action'
        body = {'os-retype': {'new_type': '1', 'migration_policy': '2'}}
        self.sess.post.assert_called_with(url, json=body)

    def test_migrate(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.migrate(self.sess, host='1'))

        url = f'volumes/{FAKE_ID}/action'
        body = {'os-migrate_volume': {'host': '1'}}
        self.sess.post.assert_called_with(url, json=body)

    def test_migrate_flags(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(
            sot.migrate(
                self.sess, host='1', force_host_copy=True, lock_volume=True
            )
        )

        url = f'volumes/{FAKE_ID}/action'
        body = {
            'os-migrate_volume': {
                'host': '1',
                'force_host_copy': True,
                'lock_volume': True,
            }
        }
        self.sess.post.assert_called_with(url, json=body)

    def test_complete_migration(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.complete_migration(self.sess, new_volume_id='1'))

        url = f'volumes/{FAKE_ID}/action'
        body = {
            'os-migrate_volume_completion': {'new_volume': '1', 'error': False}
        }
        self.sess.post.assert_called_with(url, json=body)

    def test_complete_migration_error(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(
            sot.complete_migration(self.sess, new_volume_id='1', error=True)
        )

        url = f'volumes/{FAKE_ID}/action'
        body = {
            'os-migrate_volume_completion': {'new_volume': '1', 'error': True}
        }
        self.sess.post.assert_called_with(url, json=body)

    def test_force_delete(self):
        sot = volume.Volume(**VOLUME)

        self.assertIsNone(sot.force_delete(self.sess))

        url = f'volumes/{FAKE_ID}/action'
        body = {'os-force_delete': None}
        self.sess.post.assert_called_with(url, json=body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack import resource


class Limit(resource.Resource):
    resource_key = 'limit'
    resources_key = 'limits'
    base_path = '/limits'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    commit_method = 'PATCH'
    commit_jsonpatch = True

    _query_mapping = resource.QueryParameters(
        'service_id', 'region_id', 'resource_name', 'project_id'
    )

    # Properties
    #: User-facing description of the registered_limit. *Type: string*
    description = resource.Body('description')
    #: The links for the registered_limit resource.
    links = resource.Body('links')
    #: ID of service. *Type: string*
    service_id = resource.Body('service_id')
    #: ID of region, if any. *Type: string*
    region_id = resource.Body('region_id')
    #: The resource name. *Type: string*
    resource_name = resource.Body('resource_name')
    #: The resource limit value. *Type: int*
    resource_limit = resource.Body('resource_limit')
    #: ID of project. *Type: string*
    project_id = resource.Body('project_id')

    def create(
        self,
        session,
        prepend_key=True,
        base_path=None,
        *,
        resource_request_key=None,
        resource_response_key='limits',
        microversion=None,
        **params,
    ):
        return super().create(
            session,
            prepend_key=prepend_key,
            base_path=base_path,
            resource_request_key=resource_request_key,
            resource_response_key=resource_response_key,
            microversion=microversion,
            **params,
        )

    def _prepare_request_body(
        self,
        patch,
        prepend_key,
        *,
        resource_request_key=None,
    ):
        body = self._body.dirty
        if prepend_key and self.resource_key is not None:
            if patch:
                body = {self.resource_key: body}
            else:
                # Keystone support bunch create for unified limit. So the
                # request body for creating limit is a list instead of dict.
                body = {self.resources_key: [body]}
        return body

    def _translate_response(
        self,
        response,
        has_body=None,
        error_message=None,
        *,
        resource_response_key=None,
    ):
        """Given a KSA response, inflate this instance with its data

        DELETE operations don't return a body, so only try to work
        with a body when has_body is True.

        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        if has_body is None:
            has_body = self.has_body

        exceptions.raise_from_response(response, error_message=error_message)

        if has_body:
            try:
                body = response.json()
                if resource_response_key and resource_response_key in body:
                    body = body[resource_response_key]
                elif self.resource_key and self.resource_key in body:
                    body = body[self.resource_key]

                # Keystone support bunch create for unified limit. So the
                # response body for creating limit is a list instead of dict.
                if isinstance(body, list):
                    body = body[0]

                # Do not allow keys called "self" through. Glance chose
                # to name a key "self", so we need to pop it out because
                # we can't send it through cls.existing and into the
                # Resource initializer. "self" is already the first
                # argument and is practically a reserved word.
                body.pop("self", None)

                body_attrs = self._consume_body_attrs(body)
                if self._allow_unknown_attrs_in_body:
                    body_attrs.update(body)
                    self._unknown_attrs_in_body.update(body)
                elif self._store_unknown_attrs_as_properties:
                    body_attrs = self._pack_attrs_under_properties(
                        body_attrs, body
                    )

                self._body.attributes.update(body_attrs)
                self._body.clean()
                if self.commit_jsonpatch or self.allow_patch:
                    # We need the original body to compare against
                    self._original_body = body_attrs.copy()
            except ValueError:
                # Server returned not parse-able response (202, 204, etc)
                # Do simply nothing
                pass

        headers = self._consume_header_attrs(response.headers)
        self._header.attributes.update(headers)
        self._header.clean()
        self._update_location()
        dict.update(self, self.to_dict())  # type: ignore



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class AccessRule(resource.Resource):
    resource_key = 'access_rule'
    resources_key = 'access_rules'
    base_path = '/users/%(user_id)s/access_rules'

    # capabilities
    allow_fetch = True
    allow_delete = True
    allow_list = True

    # Properties
    #: The links for the access rule resource.
    links = resource.Body('links')
    #: Method that application credential is permitted to use.
    # *Type: string*
    method = resource.Body('method')
    #: Path that the application credential is permitted to access.
    # *Type: string*
    path = resource.Body('path')
    #: Service type identifier that application credential had access.
    # *Type: string*
    service = resource.Body('service')
    #: User ID using access rule. *Type: string*
    user_id = resource.URI('user_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may

# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.object_store.v1 import _base
from openstack import resource


class Account(_base.BaseResource):
    _custom_metadata_prefix = "X-Account-Meta-"

    base_path = "/"

    allow_fetch = True
    allow_commit = True
    allow_head = True

    #: The total number of bytes that are stored in Object Storage for
    #: the account.
    account_bytes_used = resource.Header("x-account-bytes-used", type=int)
    #: The number of containers.
    account_container_count = resource.Header(
        "x-account-container-count", type=int
    )
    #: The number of objects in the account.
    account_object_count = resource.Header("x-account-object-count", type=int)
    #: The secret key value for temporary URLs. If not set,
    #: this header is not returned by this operation.
    meta_temp_url_key = resource.Header("x-account-meta-temp-url-key")
    #: A second secret key value for temporary URLs. If not set,
    #: this header is not returned by this operation.
    meta_temp_url_key_2 = resource.Header("x-account-meta-temp-url-key-2")
    #: The timestamp of the transaction.
    timestamp = resource.Header("x-timestamp")

    has_body = False
    requires_id = False

    def set_temp_url_key(self, proxy, key, secondary=False):
        """Set the temporary url key for the account.

        :param proxy: The proxy to use for making this request.
        :type proxy: :class:`~openstack.proxy.Proxy`
        :param key:
          Text of the key to use.
        :param bool secondary:
          Whether this should set the secondary key. (defaults to False)
        """
        header = 'Temp-URL-Key'
        if secondary:
            header += '-2'

        return self.set_metadata(proxy, {header: key})



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.dns.v2 import tsigkey
from openstack.tests.unit import base

IDENTIFIER = '4c72c7d3-6cfa-4fe1-9984-7705119f0228'
EXAMPLE = {
    "id": IDENTIFIER,
    "name": 'test-key',
    "algorithm": 'hmac-sha512',
    "secret": 'test-secret',
    "scope": 'POOL',
    "resource_id": IDENTIFIER,
}


class TestTsigKey(base.TestCase):
    def test_basic(self):
        sot = tsigkey.TSIGKey()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual('tsigkeys', sot.resources_key)
        self.assertEqual('/tsigkeys', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertEqual('PATCH', sot.commit_method)

        self.assertDictEqual(
            {
                'name': 'name',
                'algorithm': 'algorithm',
                'scope': 'scope',
                'limit': 'limit',
                'marker': 'marker',
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = tsigkey.TSIGKey(**EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['algorithm'], sot.algorithm)
        self.assertEqual(EXAMPLE['scope'], sot.scope)
        self.assertEqual(EXAMPLE['resource_id'], sot.resource_id)
        self.assertEqual(EXAMPLE['secret'], sot.secret)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

from openstack.orchestration.v1 import stack_template
from openstack.tests.unit import base


FAKE = {
    'description': 'template description',
    'heat_template_version': '2014-10-16',
    'parameters': {'key_name': {'type': 'string'}},
    'resources': {'resource1': {'type': 'ResourceType'}},
    'conditions': {'cd1': True},
    'outputs': {'key1': 'value1'},
}


class TestStackTemplate(base.TestCase):
    def test_basic(self):
        sot = stack_template.StackTemplate()
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)

    def test_make_it(self):
        sot = stack_template.StackTemplate(**FAKE)
        self.assertEqual(FAKE['description'], sot.description)
        self.assertEqual(
            FAKE['heat_template_version'], sot.heat_template_version
        )
        self.assertEqual(FAKE['outputs'], sot.outputs)
        self.assertEqual(FAKE['parameters'], sot.parameters)
        self.assertEqual(FAKE['resources'], sot.resources)
        self.assertEqual(FAKE['conditions'], sot.conditions)

    def test_to_dict(self):
        fake_sot = copy.deepcopy(FAKE)
        fake_sot['parameter_groups'] = [
            {
                "description": "server parameters",
                "parameters": ["key_name", "image_id"],
                "label": "server_parameters",
            }
        ]
        fake_sot['location'] = None
        fake_sot['id'] = None
        fake_sot['name'] = None

        for temp_version in [
            '2016-10-14',
            '2017-02-24',
            '2017-02-24',
            '2017-09-01',
            '2018-03-02',
            'newton',
            'ocata',
            'pike',
            'queens',
        ]:
            fake_sot['heat_template_version'] = temp_version
            sot = stack_template.StackTemplate(**fake_sot)
            self.assertEqual(fake_sot, sot.to_dict())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.placement.v1 import resource_provider_inventory
from openstack.tests.unit import base

FAKE = {
    'allocation_ratio': 1.0,
    'max_unit': 35,
    'min_unit': 1,
    'reserved': 0,
    'step_size': 1,
    'total': 35,
}


class TestResourceProviderInventory(base.TestCase):
    def test_basic(self):
        sot = resource_provider_inventory.ResourceProviderInventory()
        self.assertIsNone(sot.resource_key)
        self.assertIsNone(sot.resources_key)
        self.assertEqual(
            '/resource_providers/%(resource_provider_id)s/inventories',
            sot.base_path,
        )
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_patch)

        self.assertDictEqual({}, sot._query_mapping._mapping)

    def test_make_it(self):
        sot = resource_provider_inventory.ResourceProviderInventory(**FAKE)
        self.assertEqual(FAKE['allocation_ratio'], sot.allocation_ratio)
        self.assertEqual(FAKE['max_unit'], sot.max_unit)
        self.assertEqual(FAKE['min_unit'], sot.min_unit)
        self.assertEqual(FAKE['reserved'], sot.reserved)
        self.assertEqual(FAKE['step_size'], sot.step_size)
        self.assertEqual(FAKE['total'], sot.total)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import flavor
from openstack.tests.functional import base


class TestFlavor(base.BaseFunctionalTest):
    UPDATE_NAME = "UPDATED-NAME"
    SERVICE_TYPE = "FLAVORS"
    ID = None

    SERVICE_PROFILE_DESCRIPTION = "DESCRIPTION"
    METAINFO = "FlAVOR_PROFILE_METAINFO"

    def setUp(self):
        super().setUp()
        if not self.user_cloud._has_neutron_extension("flavors"):
            self.skipTest("Neutron flavor extension is required for this test")

        self.FLAVOR_NAME = self.getUniqueString("flavor")
        if self.operator_cloud:
            flavors = self.operator_cloud.network.create_flavor(
                name=self.FLAVOR_NAME, service_type=self.SERVICE_TYPE
            )
            assert isinstance(flavors, flavor.Flavor)
            self.assertEqual(self.FLAVOR_NAME, flavors.name)
            self.assertEqual(self.SERVICE_TYPE, flavors.service_type)

            self.ID = flavors.id

            self.service_profiles = (
                self.operator_cloud.network.create_service_profile(
                    description=self.SERVICE_PROFILE_DESCRIPTION,
                    metainfo=self.METAINFO,
                )
            )

    def tearDown(self):
        if self.operator_cloud and self.ID:
            flavors = self.operator_cloud.network.delete_flavor(
                self.ID, ignore_missing=True
            )
            self.assertIsNone(flavors)

            service_profiles = self.user_cloud.network.delete_service_profile(
                self.ID, ignore_missing=True
            )
            self.assertIsNone(service_profiles)
        super().tearDown()

    def test_find(self):
        if self.ID:
            flavors = self.user_cloud.network.find_flavor(self.FLAVOR_NAME)
            self.assertEqual(self.ID, flavors.id)
        else:
            self.user_cloud.network.find_flavor("definitely_missing")

    def test_get(self):
        if not self.ID:
            self.skipTest("Operator cloud required for this test")

        flavors = self.user_cloud.network.get_flavor(self.ID)
        self.assertEqual(self.FLAVOR_NAME, flavors.name)
        self.assertEqual(self.ID, flavors.id)

    def test_list(self):
        names = [f.name for f in self.user_cloud.network.flavors()]
        if self.ID:
            self.assertIn(self.FLAVOR_NAME, names)

    def test_update(self):
        if not self.operator_cloud:
            self.skipTest("Operator cloud required for this test")
        flavor = self.operator_cloud.network.update_flavor(
            self.ID, name=self.UPDATE_NAME
        )
        self.assertEqual(self.UPDATE_NAME, flavor.name)

    def test_associate_disassociate_flavor_with_service_profile(self):
        if not self.operator_cloud:
            self.skipTest("Operator cloud required for this test")
        response = (
            self.operator_cloud.network.associate_flavor_with_service_profile(
                self.ID, self.service_profiles.id
            )
        )
        self.assertIsNotNone(response)

        response = self.operator_cloud.network.disassociate_flavor_from_service_profile(  # noqa: E501
            self.ID, self.service_profiles.id
        )
        self.assertIsNone(response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from keystoneauth1 import adapter

from openstack.block_storage.v3 import default_type
from openstack.tests.unit import base


PROJECT_ID = 'd5e678b5-f88b-411c-876b-f6ec2ba999bf'
VOLUME_TYPE_ID = 'adef1cf8-736e-4b62-a2db-f8b6b6c1d953'

DEFAULT_TYPE = {
    'project_id': PROJECT_ID,
    'volume_type_id': VOLUME_TYPE_ID,
}


class TestDefaultType(base.TestCase):
    def setUp(self):
        super().setUp()
        self.resp = mock.Mock()
        self.resp.body = None
        self.resp.status_code = 200
        self.resp.json = mock.Mock(return_value=self.resp.body)
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = '3.67'
        self.sess.post = mock.Mock(return_value=self.resp)
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

    def test_basic(self):
        sot = default_type.DefaultType(**DEFAULT_TYPE)
        self.assertEqual("default_type", sot.resource_key)
        self.assertEqual("default_types", sot.resources_key)
        self.assertEqual("/default-types", sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_create(self):
        sot = default_type.DefaultType(**DEFAULT_TYPE)
        self.assertEqual(DEFAULT_TYPE["project_id"], sot.project_id)
        self.assertEqual(DEFAULT_TYPE["volume_type_id"], sot.volume_type_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import load_balancer
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'admin_state_up': True,
    'description': '2',
    'id': IDENTIFIER,
    'listeners': [{'id', '4'}],
    'name': '5',
    'operating_status': '6',
    'provisioning_status': '7',
    'project_id': '8',
    'vip_address': '9',
    'vip_subnet_id': '10',
    'vip_port_id': '11',
    'provider': '12',
    'pools': [{'id', '13'}],
}


class TestLoadBalancer(base.TestCase):
    def test_basic(self):
        sot = load_balancer.LoadBalancer()
        self.assertEqual('loadbalancer', sot.resource_key)
        self.assertEqual('loadbalancers', sot.resources_key)
        self.assertEqual('/lbaas/loadbalancers', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = load_balancer.LoadBalancer(**EXAMPLE)
        self.assertTrue(sot.is_admin_state_up)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['listeners'], sot.listener_ids)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['operating_status'], sot.operating_status)
        self.assertEqual(
            EXAMPLE['provisioning_status'], sot.provisioning_status
        )
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['vip_address'], sot.vip_address)
        self.assertEqual(EXAMPLE['vip_subnet_id'], sot.vip_subnet_id)
        self.assertEqual(EXAMPLE['vip_port_id'], sot.vip_port_id)
        self.assertEqual(EXAMPLE['provider'], sot.provider)
        self.assertEqual(EXAMPLE['pools'], sot.pool_ids)



# Apache 2 header omitted for brevity

from openstack import resource


class Fake(resource.Resource):
    resource_key = "resource"
    resources_key = "resources"
    base_path = "/fake"

    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    allow_head = True

    #: The transaction date and time.
    timestamp = resource.Header("x-timestamp")
    #: The name of this resource.
    name = resource.Body("name", alternate_id=True)
    #: The value of the resource. Also available in headers.
    value = resource.Body("value", alias="x-resource-value")
    #: Is this resource cool? If so, set it to True.
    #: This is a multi-line comment about cool stuff.
    cool = resource.Body("cool", type=bool)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
List resources from the Key Manager service.
"""

s = None


def get_secret_payload(conn):
    print("Get a secret's payload:")

    # Assuming you have an object `s` which you perhaps received from
    # a conn.key_manager.secrets() call...
    secret = conn.key_manager.get_secret(s.secret_id)
    print(secret.payload)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.network.v2 import _base
from openstack import resource


class DefaultSecurityGroupRule(_base.NetworkResource):
    resource_key = 'default_security_group_rule'
    resources_key = 'default_security_group_rules'
    base_path = '/default-security-group-rules'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = False
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'id',
        'description',
        'remote_group_id',
        'remote_address_group_id',
        'direction',
        'protocol',
        'port_range_min',
        'port_range_max',
        'remote_ip_prefix',
        'used_in_default_sg',
        'used_in_non_default_sg',
        'sort_dir',
        'sort_key',
        ether_type='ethertype',
    )

    # Properties
    #: The default security group rule description.
    description = resource.Body('description')
    #: The remote security group ID to be associated with this security
    #: group rule created from this template.
    #: You can specify either ``remote_group_id`` or #:
    #: ``remote_address_group_id`` or ``remote_ip_prefix``.
    remote_group_id = resource.Body('remote_group_id')
    #: The remote address group ID to be associated with this security
    #: group rule created from that template.
    #: You can specify either ``remote_group_id`` or
    #: ``remote_address_group_id`` or ``remote_ip_prefix``.
    remote_address_group_id = resource.Body('remote_address_group_id')
    #: ``ingress`` or ``egress``: The direction in which the security group #:
    #: rule will be applied. See 'direction' field in the security group rule
    #: API.
    direction = resource.Body('direction')
    #: The protocol that is matched by the security group rule.
    #: Valid values are ``null``, ``tcp``, ``udp``, and ``icmp``.
    protocol = resource.Body('protocol')
    #: The minimum port number in the range that is matched by the
    #: security group rule. If the protocol is TCP or UDP, this value
    #: must be less than or equal to the value of the port_range_max
    #: attribute. If the protocol is ICMP, this value must be an ICMP type.
    port_range_min = resource.Body('port_range_min', type=int)
    #: The maximum port number in the range that is matched by the
    #: security group rule. The port_range_min attribute constrains
    #: the port_range_max attribute. If the protocol is ICMP, this
    #: value must be an ICMP type.
    port_range_max = resource.Body('port_range_max', type=int)
    #: The remote IP prefix to be associated with this security group rule.
    #: You can specify either ``remote_group_id`` or
    #: ``remote_address_group_id`` or ``remote_ip_prefix``.
    #: This attribute matches the specified IP prefix as the source or
    #: destination IP address of the IP packet depending on direction.
    remote_ip_prefix = resource.Body('remote_ip_prefix')
    #: Must be IPv4 or IPv6, and addresses represented in CIDR must match
    #: the ingress or egress rules.
    ether_type = resource.Body('ethertype')
    #: Indicate if this template be used to create security group rules in the
    #: default security group created automatically for each project.
    used_in_default_sg = resource.Body('used_in_default_sg', type=bool)
    #: Indicate if this template be used to create security group rules in the
    #: custom security groups created in the project by users.
    used_in_non_default_sg = resource.Body('used_in_non_default_sg', type=bool)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack.load_balancer.v2 import l7_policy
from openstack.tests.unit import base


EXAMPLE = {
    'action': 'REJECT',
    'admin_state_up': True,
    'created_at': '2017-07-17T12:14:57.233772',
    'description': 'test_description',
    'id': uuid.uuid4(),
    'listener_id': uuid.uuid4(),
    'name': 'test_l7_policy',
    'operating_status': 'ONLINE',
    'position': 7,
    'project_id': uuid.uuid4(),
    'provisioning_status': 'ACTIVE',
    'redirect_pool_id': uuid.uuid4(),
    'redirect_prefix': 'https://www.example.com',
    'redirect_url': '/test_url',
    'rules': [{'id': uuid.uuid4()}],
    'updated_at': '2017-07-17T12:16:57.233772',
}


class TestL7Policy(base.TestCase):
    def test_basic(self):
        test_l7_policy = l7_policy.L7Policy()
        self.assertEqual('l7policy', test_l7_policy.resource_key)
        self.assertEqual('l7policies', test_l7_policy.resources_key)
        self.assertEqual('/lbaas/l7policies', test_l7_policy.base_path)
        self.assertTrue(test_l7_policy.allow_create)
        self.assertTrue(test_l7_policy.allow_fetch)
        self.assertTrue(test_l7_policy.allow_commit)
        self.assertTrue(test_l7_policy.allow_delete)
        self.assertTrue(test_l7_policy.allow_list)

    def test_make_it(self):
        test_l7_policy = l7_policy.L7Policy(**EXAMPLE)
        self.assertTrue(test_l7_policy.is_admin_state_up)
        self.assertEqual(EXAMPLE['action'], test_l7_policy.action)
        self.assertEqual(EXAMPLE['created_at'], test_l7_policy.created_at)
        self.assertEqual(EXAMPLE['description'], test_l7_policy.description)
        self.assertEqual(EXAMPLE['id'], test_l7_policy.id)
        self.assertEqual(EXAMPLE['listener_id'], test_l7_policy.listener_id)
        self.assertEqual(EXAMPLE['name'], test_l7_policy.name)
        self.assertEqual(
            EXAMPLE['operating_status'], test_l7_policy.operating_status
        )
        self.assertEqual(EXAMPLE['position'], test_l7_policy.position)
        self.assertEqual(EXAMPLE['project_id'], test_l7_policy.project_id)
        self.assertEqual(
            EXAMPLE['provisioning_status'], test_l7_policy.provisioning_status
        )
        self.assertEqual(
            EXAMPLE['redirect_pool_id'], test_l7_policy.redirect_pool_id
        )
        self.assertEqual(
            EXAMPLE['redirect_prefix'], test_l7_policy.redirect_prefix
        )
        self.assertEqual(EXAMPLE['redirect_url'], test_l7_policy.redirect_url)
        self.assertEqual(EXAMPLE['rules'], test_l7_policy.rules)
        self.assertEqual(EXAMPLE['updated_at'], test_l7_policy.updated_at)

        self.assertDictEqual(
            {
                'limit': 'limit',
                'marker': 'marker',
                'name': 'name',
                'description': 'description',
                'project_id': 'project_id',
                'tags': 'tags',
                'any_tags': 'tags-any',
                'not_tags': 'not-tags',
                'not_any_tags': 'not-tags-any',
                'operating_status': 'operating_status',
                'provisioning_status': 'provisioning_status',
                'is_admin_state_up': 'admin_state_up',
                'action': 'action',
                'listener_id': 'listener_id',
                'position': 'position',
                'redirect_pool_id': 'redirect_pool_id',
                'redirect_url': 'redirect_url',
                'redirect_prefix': 'redirect_prefix',
            },
            test_l7_policy._query_mapping._mapping,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import enum
import typing as ty

from keystoneauth1 import adapter as ksa_adapter

from openstack import exceptions
from openstack import resource
from openstack import utils


class Level(enum.Enum):
    ERROR = 'ERROR'
    WARNING = 'WARNING'
    INFO = 'INFO'
    DEBUG = 'DEBUG'


class Binary(enum.Enum):
    ANY = '*'
    API = 'cinder-api'
    VOLUME = 'cinder-volume'
    SCHEDULER = 'cinder-scheduler'
    BACKUP = 'cinder-backup'


class LogLevel(resource.Resource):
    # Properties
    #: The binary name of the service.
    binary = resource.Body('binary')
    # TODO(stephenfin): Do we need these? They are request-only.
    # #: The name of the host.
    # server = resource.Body('server')
    # #: he prefix for the log path we are querying, for example ``cinder.`` or
    # #: ``sqlalchemy.engine.`` When not present or the empty string is passed
    # #: all log levels will be retrieved.
    # prefix = resource.Body('prefix')
    #: The name of the host.
    host = resource.Body('host')
    #: The current log level that queried.
    levels = resource.Body('levels', type=dict)


class Service(resource.Resource):
    resources_key = 'services'
    base_path = '/os-services'

    # capabilities
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'binary',
        'host',
    )

    # Properties
    #: The ID of active storage backend (cinder-volume services only)
    active_backend_id = resource.Body('active_backend_id')
    #: The availability zone of service
    availability_zone = resource.Body('zone')
    #: The state of storage backend (cinder-volume services only) (since 3.49)
    backend_state = resource.Body('backend_state')
    #: Binary name of service
    binary = resource.Body('binary')
    #: The cluster name (since 3.7)
    cluster = resource.Body('cluster')
    #: Disabled reason of service
    disabled_reason = resource.Body('disabled_reason')
    #: The name of the host where service runs
    host = resource.Body('host')
    # Whether the host is frozen or not (cinder-volume services only)
    is_frozen = resource.Body('frozen')
    #: Service name
    name = resource.Body('name', alias='binary')
    #: The volume service replication status (cinder-volume services only)
    replication_status = resource.Body('replication_status')
    #: State of service
    state = resource.Body('state')
    #: Status of service
    status = resource.Body('status')
    #: The date and time when the resource was updated
    updated_at = resource.Body('updated_at')

    # 3.32 introduced the 'set-log' action
    _max_microversion = '3.32'

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=True, **params):
        # No direct request possible, thus go directly to list
        data = cls.list(session, **params)

        result = None
        for maybe_result in data:
            # Since ID might be both int and str force cast
            id_value = str(cls._get_id(maybe_result))
            name_value = maybe_result.name

            if str(name_or_id) in (id_value, name_value):
                if 'host' in params and maybe_result['host'] != params['host']:
                    continue
                # Only allow one resource to be found. If we already
                # found a match, raise an exception to show it.
                if result is None:
                    result = maybe_result
                else:
                    msg = "More than one %s exists with the name '%s'."
                    msg = msg % (cls.__name__, name_or_id)
                    raise exceptions.DuplicateResource(msg)

        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.NotFoundException(
            f"No {cls.__name__} found for {name_or_id}"
        )

    def commit(self, session, prepend_key=False, *args, **kwargs):
        # we need to set prepend_key to false
        return super().commit(
            session,
            prepend_key,
            *args,
            **kwargs,
        )

    def _action(self, session, action, body, microversion=None):
        if not microversion:
            microversion = session.default_microversion
        url = utils.urljoin(Service.base_path, action)
        response = session.put(url, json=body, microversion=microversion)
        self._translate_response(response)
        return self

    def enable(self, session):
        """Enable service."""
        body = {'binary': self.binary, 'host': self.host}
        return self._action(session, 'enable', body)

    def disable(self, session, *, reason=None):
        """Disable service."""
        body = {'binary': self.binary, 'host': self.host}

        if not reason:
            action = 'disable'
        else:
            action = 'disable-log-reason'
            body['disabled_reason'] = reason

        return self._action(session, action, body)

    def thaw(self, session):
        body = {'host': self.host}
        return self._action(session, 'thaw', body)

    def freeze(self, session):
        body = {'host': self.host}
        return self._action(session, 'freeze', body)

    @classmethod
    def set_log_levels(
        cls,
        session: ksa_adapter.Adapter,
        *,
        level: Level,
        binary: Binary | None = None,
        server: str | None = None,
        prefix: str | None = None,
    ) -> None:
        """Set log level for services.

        :param session: The session to use for making this request.
        :param level: The log level to set, case insensitive, accepted values
            are ``INFO``, ``WARNING``, ``ERROR`` and ``DEBUG``.
        :param binary: The binary name of the service.
        :param server: The name of the host.
        :param prefix: The prefix for the log path we are querying, for example
            ``cinder.`` or ``sqlalchemy.engine.`` When not present or the empty
            string is passed all log levels will be retrieved.
        :returns: None.
        """
        microversion = cls._assert_microversion_for(
            session, '3.32', error_message="Cannot use set-log action"
        )
        body = {
            'level': level,
            'binary': binary or '',  # cinder insists on strings
            'server': server,
            'prefix': prefix,
        }
        url = utils.urljoin(cls.base_path, 'set-log')
        response = session.put(url, json=body, microversion=microversion)
        exceptions.raise_from_response(response)

    @classmethod
    def get_log_levels(
        cls,
        session: ksa_adapter.Adapter,
        *,
        binary: Binary | None = None,
        server: str | None = None,
        prefix: str | None = None,
    ) -> ty.Generator[LogLevel, None, None]:
        """Get log level for services.

        :param session: The session to use for making this request.
        :param binary: The binary name of the service.
        :param server: The name of the host.
        :param prefix: The prefix for the log path we are querying, for example
            ``cinder.`` or ``sqlalchemy.engine.`` When not present or the empty
            string is passed all log levels will be retrieved.
        :returns: A generator of
            :class:`~openstack.block_storage.v3.service.LogLevel` objects.
        """
        microversion = cls._assert_microversion_for(
            session, '3.32', error_message="Cannot use get-log action"
        )
        body = {
            'binary': binary or '',  # cinder insists on strings
            'server': server,
            'prefix': prefix,
        }
        url = utils.urljoin(cls.base_path, 'get-log')
        response = session.put(url, json=body, microversion=microversion)
        exceptions.raise_from_response(response)

        for entry in response.json()['log_levels']:
            yield LogLevel(
                binary=entry['binary'],
                host=entry['host'],
                levels=entry['levels'],
            )

    def failover(
        self,
        session,
        *,
        cluster=None,
        backend_id=None,
    ):
        """Failover a service

        Only applies to replicating cinder-volume services.
        """
        body = {'host': self.host}
        if cluster:
            body['cluster'] = cluster
        if backend_id:
            body['backend_id'] = backend_id

        action = 'failover_host'
        if utils.supports_microversion(session, '3.26'):
            action = 'failover'

        return self._action(session, action, body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.network.v2 import address_scope as _address_scope
from openstack.tests.functional import base


class TestAddressScope(base.BaseFunctionalTest):
    ADDRESS_SCOPE_ID = None
    IS_SHARED = False
    IP_VERSION = 4

    def setUp(self):
        super().setUp()
        self.ADDRESS_SCOPE_NAME = self.getUniqueString()
        self.ADDRESS_SCOPE_NAME_UPDATED = self.getUniqueString()
        address_scope = self.user_cloud.network.create_address_scope(
            ip_version=self.IP_VERSION,
            name=self.ADDRESS_SCOPE_NAME,
            shared=self.IS_SHARED,
        )
        assert isinstance(address_scope, _address_scope.AddressScope)
        self.assertEqual(self.ADDRESS_SCOPE_NAME, address_scope.name)
        self.ADDRESS_SCOPE_ID = address_scope.id

    def tearDown(self):
        sot = self.user_cloud.network.delete_address_scope(
            self.ADDRESS_SCOPE_ID
        )
        self.assertIsNone(sot)
        super().tearDown()

    def test_find(self):
        sot = self.user_cloud.network.find_address_scope(
            self.ADDRESS_SCOPE_NAME
        )
        self.assertEqual(self.ADDRESS_SCOPE_ID, sot.id)

    def test_get(self):
        sot = self.user_cloud.network.get_address_scope(self.ADDRESS_SCOPE_ID)
        self.assertEqual(self.ADDRESS_SCOPE_NAME, sot.name)
        self.assertEqual(self.IS_SHARED, sot.is_shared)
        self.assertEqual(self.IP_VERSION, sot.ip_version)

    def test_list(self):
        names = [o.name for o in self.user_cloud.network.address_scopes()]
        self.assertIn(self.ADDRESS_SCOPE_NAME, names)

    def test_update(self):
        sot = self.user_cloud.network.update_address_scope(
            self.ADDRESS_SCOPE_ID, name=self.ADDRESS_SCOPE_NAME_UPDATED
        )
        self.assertEqual(self.ADDRESS_SCOPE_NAME_UPDATED, sot.name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.dns.v2 import _base
from openstack import exceptions
from openstack import resource


class ZoneImport(_base.Resource):
    """DNS Zone Import Resource"""

    resource_key = ''
    resources_key = 'imports'
    base_path = '/zones/tasks/import'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters('zone_id', 'message', 'status')

    #: Properties
    #: Timestamp when the zone was created
    created_at = resource.Body('created_at')
    #: Links contains a `self` pertaining to this zone or a `next` pertaining
    #: to next page
    links = resource.Body('links', type=dict)
    #: Message
    message = resource.Body('message')
    #: Returns the total_count of resources matching this filter
    metadata = resource.Body('metadata', type=list)
    #: The project id which the zone belongs to
    project_id = resource.Body('project_id')
    #: Current status of the zone import
    status = resource.Body('status')
    #: Timestamp when the zone was last updated
    updated_at = resource.Body('updated_at')
    #: Version of the resource
    version = resource.Body('version', type=int)
    #: ID for the zone that was created by this import
    zone_id = resource.Body('zone_id')

    def create(self, session, prepend_key=True, base_path=None, **kwargs):
        """Create a remote resource based on this instance.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param prepend_key: A boolean indicating whether the resource_key
                            should be prepended in a resource creation
                            request. Default to True.
        :param str base_path: Base part of the URI for creating resources, if
                              different from
                              :data:`~openstack.resource.Resource.base_path`.
        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_create` is not set to ``True``.
        """
        if not self.allow_create:
            raise exceptions.MethodNotSupported(self, "create")

        session = self._get_session(session)
        microversion = self._get_microversion(session)
        # Create ZoneImport requires empty body and 'text/dns' as content-type
        # skip _prepare_request completely, since we need just empty body
        request = resource._Request(
            self.base_path, None, {'content-type': 'text/dns'}
        )
        response = session.post(
            request.url,
            json=request.body,
            headers=request.headers,
            microversion=microversion,
        )

        self.microversion = microversion
        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from keystoneauth1 import adapter

from openstack.block_storage.v3 import group_type
from openstack.tests.unit import base

GROUP_TYPE = {
    "id": "6685584b-1eac-4da6-b5c3-555430cf68ff",
    "name": "grp-type-001",
    "description": "group type 001",
    "is_public": True,
    "group_specs": {"consistent_group_snapshot_enabled": "<is> False"},
}


class TestGroupType(base.TestCase):
    def setUp(self):
        super().setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = 1
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

    def test_basic(self):
        resource = group_type.GroupType()
        self.assertEqual("group_type", resource.resource_key)
        self.assertEqual("group_types", resource.resources_key)
        self.assertEqual("/group_types", resource.base_path)
        self.assertTrue(resource.allow_create)
        self.assertTrue(resource.allow_fetch)
        self.assertTrue(resource.allow_delete)
        self.assertTrue(resource.allow_commit)
        self.assertTrue(resource.allow_list)

    def test_make_resource(self):
        resource = group_type.GroupType(**GROUP_TYPE)
        self.assertEqual(GROUP_TYPE["id"], resource.id)
        self.assertEqual(GROUP_TYPE["name"], resource.name)
        self.assertEqual(GROUP_TYPE["description"], resource.description)
        self.assertEqual(GROUP_TYPE["is_public"], resource.is_public)
        self.assertEqual(GROUP_TYPE["group_specs"], resource.group_specs)

    def test_fetch_group_specs(self):
        sot = group_type.GroupType(**GROUP_TYPE)
        resp = mock.Mock()
        resp.body = {'group_specs': {'a': 'b', 'c': 'd'}}
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        self.sess.get = mock.Mock(return_value=resp)

        rsp = sot.fetch_group_specs(self.sess)

        self.sess.get.assert_called_with(
            f"group_types/{GROUP_TYPE['id']}/group_specs",
            microversion=self.sess.default_microversion,
        )

        self.assertEqual(resp.body['group_specs'], rsp.group_specs)
        self.assertIsInstance(rsp, group_type.GroupType)

    def test_create_group_specs(self):
        sot = group_type.GroupType(**GROUP_TYPE)
        specs = {'a': 'b', 'c': 'd'}
        resp = mock.Mock()
        resp.body = {'group_specs': specs}
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        self.sess.post = mock.Mock(return_value=resp)

        rsp = sot.create_group_specs(self.sess, specs)

        self.sess.post.assert_called_with(
            f"group_types/{GROUP_TYPE['id']}/group_specs",
            json={'group_specs': specs},
            microversion=self.sess.default_microversion,
        )

        self.assertEqual(resp.body['group_specs'], rsp.group_specs)
        self.assertIsInstance(rsp, group_type.GroupType)

    def test_get_group_specs_property(self):
        sot = group_type.GroupType(**GROUP_TYPE)
        resp = mock.Mock()
        resp.body = {'a': 'b'}
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        self.sess.get = mock.Mock(return_value=resp)

        rsp = sot.get_group_specs_property(self.sess, 'a')

        self.sess.get.assert_called_with(
            f"group_types/{GROUP_TYPE['id']}/group_specs/a",
            microversion=self.sess.default_microversion,
        )

        self.assertEqual('b', rsp)

    def test_update_group_specs_property(self):
        sot = group_type.GroupType(**GROUP_TYPE)
        resp = mock.Mock()
        resp.body = {'a': 'b'}
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        self.sess.put = mock.Mock(return_value=resp)

        rsp = sot.update_group_specs_property(self.sess, 'a', 'b')

        self.sess.put.assert_called_with(
            f"group_types/{GROUP_TYPE['id']}/group_specs/a",
            json={'a': 'b'},
            microversion=self.sess.default_microversion,
        )

        self.assertEqual('b', rsp)

    def test_delete_group_specs_property(self):
        sot = group_type.GroupType(**GROUP_TYPE)
        resp = mock.Mock()
        resp.body = None
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        self.sess.delete = mock.Mock(return_value=resp)

        rsp = sot.delete_group_specs_property(self.sess, 'a')

        self.sess.delete.assert_called_with(
            f"group_types/{GROUP_TYPE['id']}/group_specs/a",
            microversion=self.sess.default_microversion,
        )

        self.assertIsNone(rsp)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack.identity.v3 import domain as _domain
from openstack.identity.v3 import domain_config as _domain_config
from openstack.tests.functional import base


class TestDomainConfig(base.BaseFunctionalTest):
    def setUp(self):
        super().setUp()

        self.domain_name = self.getUniqueString()

        # create the domain and domain config

        self.domain = self.operator_cloud.create_domain(
            name=self.domain_name,
        )
        self.assertIsInstance(self.domain, _domain.Domain)
        self.addCleanup(self._delete_domain)

    def _delete_domain(self):
        self.operator_cloud.identity.update_domain(
            self.domain,
            enabled=False,
        )
        self.operator_cloud.identity.delete_domain(self.domain)

    def test_domain_config(self):
        # create the domain config

        domain_config = self.operator_cloud.identity.create_domain_config(
            self.domain,
            identity={'driver': uuid.uuid4().hex},
            ldap={'url': uuid.uuid4().hex},
        )
        self.assertIsInstance(
            domain_config,
            _domain_config.DomainConfig,
        )

        # update the domain config

        ldap_url = uuid.uuid4().hex
        domain_config = self.operator_cloud.identity.update_domain_config(
            self.domain,
            ldap={'url': ldap_url},
        )
        self.assertIsInstance(
            domain_config,
            _domain_config.DomainConfig,
        )

        # retrieve details of the (updated) domain config

        domain_config = self.operator_cloud.identity.get_domain_config(
            self.domain,
        )
        self.assertIsInstance(
            domain_config,
            _domain_config.DomainConfig,
        )
        self.assertEqual(ldap_url, domain_config.ldap.url)

        # delete the domain config

        result = self.operator_cloud.identity.delete_domain_config(
            self.domain,
            ignore_missing=False,
        )
        self.assertIsNone(result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack.tests.functional.baremetal import base


class TestBareMetalInspectionRule(base.BaseBaremetalTest):
    min_microversion = '1.96'

    def setUp(self):
        super().setUp()

    def test_baremetal_inspection_rule_create_get_delete(self):
        actions = [{"op": "set-attribute", "args": ["/driver", "idrac"]}]
        conditions = [
            {"op": "eq", "args": ["node:memory_mb", 4096], "multiple": "all"}
        ]
        inspection_rule = self.create_inspection_rule(
            actions=actions,
            conditions=conditions,
            description="Test inspection rule",
            phase="main",
            priority=100,
            sensitive=False,
        )
        loaded = self.system_admin_cloud.baremetal.get_inspection_rule(
            inspection_rule.id
        )
        self.assertEqual(loaded.id, inspection_rule.id)
        self.system_admin_cloud.baremetal.delete_inspection_rule(
            inspection_rule, ignore_missing=False
        )
        self.assertRaises(
            exceptions.NotFoundException,
            self.system_admin_cloud.baremetal.get_inspection_rule,
            inspection_rule.id,
        )

    def test_baremetal_inspection_rule_list(self):
        actions = [{"op": "set-attribute", "args": ["/driver", "idrac"]}]
        conditions = [
            {
                "op": "is-true",
                "args": ["{node.auto_discovered}"],
                "multiple": "any",
            }
        ]

        inspection_rule1 = self.create_inspection_rule(
            actions=actions,
            conditions=conditions,
            description="Test inspection rule 1",
        )
        inspection_rule2 = self.create_inspection_rule(
            actions=actions,
            conditions=conditions,
            description="Test inspection rule 2",
        )
        inspection_rules = self.system_admin_cloud.baremetal.inspection_rules()
        ids = [rule.id for rule in inspection_rules]
        self.assertIn(inspection_rule1.id, ids)
        self.assertIn(inspection_rule2.id, ids)

        inspection_rules_with_details = (
            self.system_admin_cloud.baremetal.inspection_rules(details=True)
        )
        for rule in inspection_rules_with_details:
            self.assertIsNotNone(rule.id)
            self.assertIsNotNone(rule.description)

        inspection_rule_with_fields = (
            self.system_admin_cloud.baremetal.inspection_rules(fields=['uuid'])
        )
        for rule in inspection_rule_with_fields:
            self.assertIsNotNone(rule.id)
            self.assertIsNone(rule.description)

    def test_baremetal_inspection_rule_list_update_delete(self):
        actions = [{"op": "set-attribute", "args": ["/driver", "idrac"]}]
        conditions = [
            {
                "op": "eq",
                "args": ["node:cpu_arch", "x86_64"],
                "multiple": "all",
            }
        ]
        inspection_rule = self.create_inspection_rule(
            actions=actions,
            conditions=conditions,
            description="Test inspection rule",
        )
        inspection_rule.description = 'Updated inspection rule'

        inspection_rule = (
            self.system_admin_cloud.baremetal.update_inspection_rule(
                inspection_rule
            )
        )
        self.assertEqual(
            'Updated inspection rule', inspection_rule.description
        )

        inspection_rule = (
            self.system_admin_cloud.baremetal.get_inspection_rule(
                inspection_rule.id
            )
        )

        self.system_admin_cloud.baremetal.delete_inspection_rule(
            inspection_rule.id, ignore_missing=False
        )

    def test_baremetal_inspection_rule_update(self):
        actions = [{"op": "set-attribute", "args": ["/driver", "idrac"]}]
        conditions = [
            {"op": "gt", "args": ["node:memory_mb", 4096], "multiple": "all"}
        ]
        inspection_rule = self.create_inspection_rule(
            actions=actions, conditions=conditions, phase="main", priority=100
        )
        inspection_rule.priority = 150

        inspection_rule = (
            self.system_admin_cloud.baremetal.update_inspection_rule(
                inspection_rule
            )
        )
        self.assertEqual(150, inspection_rule.priority)

        inspection_rule = (
            self.system_admin_cloud.baremetal.get_inspection_rule(
                inspection_rule.id
            )
        )
        self.assertEqual(150, inspection_rule.priority)

    def test_inspection_rule_patch(self):
        description = "BIOS configuration rule"
        actions = [
            {
                "op": "set-attribute",
                "args": ["/properties/capabilities", "boot_mode:uefi"],
            }
        ]
        conditions = [
            {
                "op": "is-true",
                "args": ["{node.auto_discovered}"],
                "multiple": "any",
            }
        ]
        inspection_rule = self.create_inspection_rule(
            actions=actions,
            conditions=conditions,
            description=description,
            sensitive=False,
        )

        updated_actions = [
            {
                "op": "set-attribute",
                "args": ["/driver", "fake"],
            }
        ]

        inspection_rule = (
            self.system_admin_cloud.baremetal.patch_inspection_rule(
                inspection_rule,
                dict(path='/actions', op='add', value=updated_actions),
            )
        )
        self.assertEqual(updated_actions, inspection_rule.actions)
        self.assertEqual(description, inspection_rule.description)

        inspection_rule = (
            self.system_admin_cloud.baremetal.get_inspection_rule(
                inspection_rule.id
            )
        )
        self.assertEqual(updated_actions, inspection_rule.actions)

    def test_inspection_rule_negative_non_existing(self):
        uuid = "bbb45f41-d4bc-4307-8d1d-32f95ce1e920"
        self.assertRaises(
            exceptions.NotFoundException,
            self.system_admin_cloud.baremetal.get_inspection_rule,
            uuid,
        )
        self.assertRaises(
            exceptions.NotFoundException,
            self.system_admin_cloud.baremetal.delete_inspection_rule,
            uuid,
            ignore_missing=False,
        )
        self.assertIsNone(
            self.system_admin_cloud.baremetal.delete_inspection_rule(uuid)
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.container_infrastructure_management.v1 import (
    cluster_certificate,
)
from openstack.container_infrastructure_management.v1 import _proxy
from openstack.container_infrastructure_management.v1 import cluster
from openstack.container_infrastructure_management.v1 import cluster_template
from openstack.container_infrastructure_management.v1 import service
from openstack.tests.unit import test_proxy_base


class TestMagnumProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super().setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestCluster(TestMagnumProxy):
    def test_cluster_get(self):
        self.verify_get(self.proxy.get_cluster, cluster.Cluster)

    def test_cluster_find(self):
        self.verify_find(
            self.proxy.find_cluster,
            cluster.Cluster,
            method_kwargs={},
            expected_kwargs={},
        )

    def test_clusters(self):
        self.verify_list(
            self.proxy.clusters,
            cluster.Cluster,
            method_kwargs={"query": 1},
            expected_kwargs={"query": 1},
        )

    def test_cluster_create_attrs(self):
        self.verify_create(self.proxy.create_cluster, cluster.Cluster)

    def test_cluster_delete(self):
        self.verify_delete(self.proxy.delete_cluster, cluster.Cluster, False)

    def test_cluster_delete_ignore(self):
        self.verify_delete(self.proxy.delete_cluster, cluster.Cluster, True)


class TestClusterCertificate(TestMagnumProxy):
    def test_cluster_certificate_get(self):
        self.verify_get(
            self.proxy.get_cluster_certificate,
            cluster_certificate.ClusterCertificate,
        )

    def test_cluster_certificate_create_attrs(self):
        self.verify_create(
            self.proxy.create_cluster_certificate,
            cluster_certificate.ClusterCertificate,
        )


class TestClusterTemplate(TestMagnumProxy):
    def test_cluster_template_get(self):
        self.verify_get(
            self.proxy.get_cluster_template, cluster_template.ClusterTemplate
        )

    def test_cluster_template_find(self):
        self.verify_find(
            self.proxy.find_cluster_template,
            cluster_template.ClusterTemplate,
            method_kwargs={},
            expected_kwargs={},
        )

    def test_cluster_templates(self):
        self.verify_list(
            self.proxy.cluster_templates,
            cluster_template.ClusterTemplate,
            method_kwargs={"query": 1},
            expected_kwargs={"query": 1},
        )

    def test_cluster_template_create_attrs(self):
        self.verify_create(
            self.proxy.create_cluster_template,
            cluster_template.ClusterTemplate,
        )

    def test_cluster_template_delete(self):
        self.verify_delete(
            self.proxy.delete_cluster_template,
            cluster_template.ClusterTemplate,
            False,
        )

    def test_cluster_template_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_cluster_template,
            cluster_template.ClusterTemplate,
            True,
        )


class TestService(TestMagnumProxy):
    def test_services(self):
        self.verify_list(
            self.proxy.services,
            service.Service,
            method_kwargs={},
            expected_kwargs={},
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class QuotaClassSet(resource.Resource):
    resource_key = 'quota_class_set'
    base_path = '/os-quota-class-sets'

    # Capabilities
    allow_fetch = True
    allow_commit = True

    # Properties
    #: The size (GB) of backups that are allowed for each project.
    backup_gigabytes = resource.Body('backup_gigabytes', type=int)
    #: The number of backups that are allowed for each project.
    backups = resource.Body('backups', type=int)
    #: The size (GB) of volumes and snapshots that are allowed for each
    #: project.
    gigabytes = resource.Body('gigabytes', type=int)
    #: The number of groups that are allowed for each project.
    groups = resource.Body('groups', type=int)
    #: The size (GB) of volumes in request that are allowed for each volume.
    per_volume_gigabytes = resource.Body('per_volume_gigabytes', type=int)
    #: The number of snapshots that are allowed for each project.
    snapshots = resource.Body('snapshots', type=int)
    #: The number of volumes that are allowed for each project.
    volumes = resource.Body('volumes', type=int)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.dns.v2 import _proxy
from openstack.dns.v2 import floating_ip
from openstack.dns.v2 import recordset
from openstack.dns.v2 import service_status
from openstack.dns.v2 import tsigkey
from openstack.dns.v2 import zone
from openstack.dns.v2 import zone_export
from openstack.dns.v2 import zone_import
from openstack.dns.v2 import zone_nameserver
from openstack.dns.v2 import zone_share
from openstack.dns.v2 import zone_transfer
from openstack.tests.unit import test_proxy_base


class TestDnsProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super().setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestDnsZone(TestDnsProxy):
    def test_zone_create(self):
        self.verify_create(
            self.proxy.create_zone,
            zone.Zone,
            method_kwargs={'name': 'id'},
            expected_kwargs={'name': 'id', 'prepend_key': False},
        )

    def test_zone_delete(self):
        self.verify_delete(
            self.proxy.delete_zone,
            zone.Zone,
            True,
            expected_kwargs={'ignore_missing': True, 'delete_shares': False},
        )

    def test_zone_find(self):
        self.verify_find(self.proxy.find_zone, zone.Zone)

    def test_zone_get(self):
        self.verify_get(self.proxy.get_zone, zone.Zone)

    def test_zones(self):
        self.verify_list(self.proxy.zones, zone.Zone)

    def test_zone_update(self):
        self.verify_update(self.proxy.update_zone, zone.Zone)

    def test_zone_abandon(self):
        self._verify(
            "openstack.dns.v2.zone.Zone.abandon",
            self.proxy.abandon_zone,
            method_args=[{'zone': 'id'}],
            expected_args=[self.proxy],
        )

    def test_zone_xfr(self):
        self._verify(
            "openstack.dns.v2.zone.Zone.xfr",
            self.proxy.xfr_zone,
            method_args=[{'zone': 'id'}],
            expected_args=[self.proxy],
        )


class TestDnsZoneNameserver(TestDnsProxy):
    def test_get_zone_nameservers(self):
        self.verify_list(
            self.proxy.zone_nameservers,
            zone_nameserver.ZoneNameserver,
            method_kwargs={'zone': 'id'},
            expected_kwargs={'zone_id': 'id'},
        )


class TestDnsRecordset(TestDnsProxy):
    def test_recordset_create(self):
        self.verify_create(
            self.proxy.create_recordset,
            recordset.Recordset,
            method_kwargs={'zone': 'id'},
            expected_kwargs={'zone_id': 'id', 'prepend_key': False},
        )

    def test_recordset_delete(self):
        self.verify_delete(
            self.proxy.delete_recordset, recordset.Recordset, True
        )

    def test_recordset_update(self):
        self.verify_update(self.proxy.update_recordset, recordset.Recordset)

    def test_recordset_get(self):
        self.verify_get(
            self.proxy.get_recordset,
            recordset.Recordset,
            method_kwargs={'zone': 'zid'},
            expected_kwargs={'zone_id': 'zid'},
        )

    def test_recordsets(self):
        self.verify_list(
            self.proxy.recordsets,
            recordset.Recordset,
            expected_kwargs={'base_path': '/recordsets'},
        )

    def test_recordsets_zone(self):
        self.verify_list(
            self.proxy.recordsets,
            recordset.Recordset,
            method_kwargs={'zone': 'zid'},
            expected_kwargs={'zone_id': 'zid'},
        )

    def test_recordset_find(self):
        self._verify(
            "openstack.proxy.Proxy._find",
            self.proxy.find_recordset,
            method_args=['zone', 'rs'],
            method_kwargs={},
            expected_args=[recordset.Recordset, 'rs'],
            expected_kwargs={'ignore_missing': True, 'zone_id': 'zone'},
        )


class TestDnsFloatIP(TestDnsProxy):
    def test_floating_ips(self):
        self.verify_list(self.proxy.floating_ips, floating_ip.FloatingIP)

    def test_floating_ip_get(self):
        self.verify_get(self.proxy.get_floating_ip, floating_ip.FloatingIP)

    def test_floating_ip_update(self):
        self.verify_update(
            self.proxy.update_floating_ip, floating_ip.FloatingIP
        )

    def test_floating_ip_unset(self):
        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.unset_floating_ip,
            method_args=['value'],
            method_kwargs={},
            expected_args=[floating_ip.FloatingIP, 'value'],
            expected_kwargs={'ptrdname': None},
        )


class TestDnsZoneImport(TestDnsProxy):
    def test_zone_import_delete(self):
        self.verify_delete(
            self.proxy.delete_zone_import, zone_import.ZoneImport, True
        )

    def test_zone_import_get(self):
        self.verify_get(self.proxy.get_zone_import, zone_import.ZoneImport)

    def test_zone_imports(self):
        self.verify_list(self.proxy.zone_imports, zone_import.ZoneImport)

    def test_zone_import_create(self):
        self.verify_create(
            self.proxy.create_zone_import,
            zone_import.ZoneImport,
            method_kwargs={'name': 'id'},
            expected_kwargs={'name': 'id', 'prepend_key': False},
        )


class TestDnsZoneExport(TestDnsProxy):
    def test_zone_export_delete(self):
        self.verify_delete(
            self.proxy.delete_zone_export, zone_export.ZoneExport, True
        )

    def test_zone_export_get(self):
        self.verify_get(self.proxy.get_zone_export, zone_export.ZoneExport)

    def test_zone_export_get_text(self):
        self.verify_get(
            self.proxy.get_zone_export_text,
            zone_export.ZoneExport,
            method_args=[{'id': 'zone_export_id_value'}],
            expected_kwargs={'base_path': '/zones/tasks/export/%(id)s/export'},
        )

    def test_zone_exports(self):
        self.verify_list(self.proxy.zone_exports, zone_export.ZoneExport)

    def test_zone_export_create(self):
        self.verify_create(
            self.proxy.create_zone_export,
            zone_export.ZoneExport,
            method_args=[{'id': 'zone_id_value'}],
            method_kwargs={'name': 'id'},
            expected_args=[],
            expected_kwargs={
                'name': 'id',
                'zone_id': 'zone_id_value',
                'prepend_key': False,
            },
        )


class TestDnsZoneTransferRequest(TestDnsProxy):
    def test_zone_transfer_request_delete(self):
        self.verify_delete(
            self.proxy.delete_zone_transfer_request,
            zone_transfer.ZoneTransferRequest,
            True,
        )

    def test_zone_transfer_request_get(self):
        self.verify_get(
            self.proxy.get_zone_transfer_request,
            zone_transfer.ZoneTransferRequest,
        )

    def test_zone_transfer_requests(self):
        self.verify_list(
            self.proxy.zone_transfer_requests,
            zone_transfer.ZoneTransferRequest,
        )

    def test_zone_transfer_request_create(self):
        self.verify_create(
            self.proxy.create_zone_transfer_request,
            zone_transfer.ZoneTransferRequest,
            method_args=[{'id': 'zone_id_value'}],
            method_kwargs={'name': 'id'},
            expected_args=[],
            expected_kwargs={
                'name': 'id',
                'zone_id': 'zone_id_value',
                'prepend_key': False,
            },
        )

    def test_zone_transfer_request_update(self):
        self.verify_update(
            self.proxy.update_zone_transfer_request,
            zone_transfer.ZoneTransferRequest,
        )


class TestDnsZoneTransferAccept(TestDnsProxy):
    def test_zone_transfer_accept_get(self):
        self.verify_get(
            self.proxy.get_zone_transfer_accept,
            zone_transfer.ZoneTransferAccept,
        )

    def test_zone_transfer_accepts(self):
        self.verify_list(
            self.proxy.zone_transfer_accepts, zone_transfer.ZoneTransferAccept
        )

    def test_zone_transfer_accept_create(self):
        self.verify_create(
            self.proxy.create_zone_transfer_accept,
            zone_transfer.ZoneTransferAccept,
        )


class TestDnsZoneShare(TestDnsProxy):
    def test_zone_share_create(self):
        self.verify_create(
            self.proxy.create_zone_share,
            zone_share.ZoneShare,
            method_kwargs={'zone': 'bogus_id'},
            expected_kwargs={'zone_id': 'bogus_id'},
        )

    def test_zone_share_delete(self):
        self.verify_delete(
            self.proxy.delete_zone_share,
            zone_share.ZoneShare,
            ignore_missing=True,
            method_args={'zone': 'bogus_id', 'zone_share': 'bogus_id'},
            expected_args=['zone_share'],
            expected_kwargs={'zone_id': 'zone', 'ignore_missing': True},
        )

    def test_zone_share_find(self):
        self.verify_find(
            self.proxy.find_zone_share,
            zone_share.ZoneShare,
            method_args=['zone'],
            expected_args=['zone'],
            expected_kwargs={
                'zone_id': 'resource_name',
                'ignore_missing': True,
            },
        )

    def test_zone_share_get(self):
        self.verify_get(
            self.proxy.get_zone_share,
            zone_share.ZoneShare,
            method_args=['zone', 'zone_share'],
            expected_args=['zone_share'],
            expected_kwargs={'zone_id': 'zone'},
        )

    def test_zone_shares(self):
        self.verify_list(
            self.proxy.zone_shares,
            zone_share.ZoneShare,
            method_args=['zone'],
            expected_args=[],
            expected_kwargs={'zone_id': 'zone'},
        )


class TestDnsServiceStatus(TestDnsProxy):
    def test_service_statuses(self):
        self.verify_list(
            self.proxy.service_statuses, service_status.ServiceStatus
        )

    def test_service_status_get(self):
        self.verify_get(
            self.proxy.get_service_status, service_status.ServiceStatus
        )


class TestDnsTsigKey(TestDnsProxy):
    def test_tsigkey_create(self):
        self.verify_create(
            self.proxy.create_tsigkey,
            tsigkey.TSIGKey,
            method_kwargs={'name': 'id'},
            expected_kwargs={'name': 'id', 'prepend_key': False},
        )

    def test_tsigkey_delete(self):
        self.verify_delete(
            self.proxy.delete_tsigkey,
            tsigkey.TSIGKey,
            True,
            expected_kwargs={'ignore_missing': True, 'delete_shares': False},
        )

    def test_tsigkey_find(self):
        self.verify_find(self.proxy.find_tsigkey, tsigkey.TSIGKey)

    def test_tsigkey_get(self):
        self.verify_get(self.proxy.get_tsigkey, tsigkey.TSIGKey)

    def test_tesigkeys(self):
        self.verify_list(self.proxy.tsigkeys, tsigkey.TSIGKey)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import typing as ty

from openstack import exceptions
from openstack.orchestration.util import template_utils
from openstack.orchestration.v1 import resource as _resource
from openstack.orchestration.v1 import software_config as _sc
from openstack.orchestration.v1 import software_deployment as _sd
from openstack.orchestration.v1 import stack as _stack
from openstack.orchestration.v1 import stack_environment as _stack_environment
from openstack.orchestration.v1 import stack_event as _stack_event
from openstack.orchestration.v1 import stack_files as _stack_files
from openstack.orchestration.v1 import stack_template as _stack_template
from openstack.orchestration.v1 import template as _template
from openstack import proxy
from openstack import resource


# TODO(rladntjr4): Some of these methods support lookup by ID, while others
# support lookup by ID or name. We should choose one and use it consistently.
class Proxy(proxy.Proxy):
    _resource_registry = {
        "resource": _resource.Resource,
        "software_config": _sc.SoftwareConfig,
        "software_deployment": _sd.SoftwareDeployment,
        "stack": _stack.Stack,
        "stack_environment": _stack_environment.StackEnvironment,
        "stack_files": _stack_files.StackFiles,
        "stack_template": _stack_template.StackTemplate,
    }

    def _extract_name_consume_url_parts(self, url_parts):
        if (
            len(url_parts) == 3
            and url_parts[0] == 'software_deployments'
            and url_parts[1] == 'metadata'
        ):
            # Another nice example of totally different URL naming scheme,
            # which we need to repair /software_deployment/metadata/server_id -
            # just replace server_id with metadata to keep further logic
            return ['software_deployment', 'metadata']
        if (
            url_parts[0] == 'stacks'
            and len(url_parts) > 2
            and url_parts[2] not in ['preview', 'resources']
        ):
            # orchestrate introduce having stack name and id part of the URL
            # (/stacks/name/id/everything_else), so if on third position we
            # have not a known part - discard it, not to brake further logic
            del url_parts[2]
        return super()._extract_name_consume_url_parts(url_parts)

    def read_env_and_templates(
        self,
        template_file=None,
        template_url=None,
        template_object=None,
        files=None,
        environment_files=None,
    ):
        """Read templates and environment content and prepares
        corresponding stack attributes

        :param string template_file: Path to the template.
        :param string template_url: URL of template.
        :param string template_object: URL to retrieve template object.
        :param dict files: dict of additional file content to include.
        :param environment_files: Paths to environment files to apply.

        :returns: Attributes dict to be set on the
            :class:`~openstack.orchestration.v1.stack.Stack`
        :rtype: dict
        """
        stack_attrs = dict()
        envfiles = dict()
        tpl_files = None
        if environment_files:
            (
                envfiles,
                env,
            ) = template_utils.process_multiple_environments_and_files(
                env_paths=environment_files
            )
            stack_attrs['environment'] = env
        if template_file or template_url or template_object:
            tpl_files, template = template_utils.get_template_contents(
                template_file=template_file,
                template_url=template_url,
                template_object=template_object,
                files=files,
            )
            stack_attrs['template'] = template
            if tpl_files or envfiles:
                stack_attrs['files'] = dict(
                    list(tpl_files.items()) + list(envfiles.items())
                )
        return stack_attrs

    def create_stack(self, preview=False, **attrs):
        """Create a new stack from attributes

        :param bool preview: When ``True``, a preview endpoint will be used to
            verify the template
            *Default: ``False``*
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.orchestration.v1.stack.Stack`,
            comprised of the properties on the Stack class.

        :returns: The results of stack creation
        :rtype: :class:`~openstack.orchestration.v1.stack.Stack`
        """

        base_path = None if not preview else '/stacks/preview'
        return self._create(_stack.Stack, base_path=base_path, **attrs)

    def find_stack(
        self, name_or_id, ignore_missing=True, resolve_outputs=True
    ):
        """Find a single stack

        :param name_or_id: The name or ID of a stack.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.orchestration.v1.stack.Stack` or None
        """
        return self._find(
            _stack.Stack,
            name_or_id,
            ignore_missing=ignore_missing,
            resolve_outputs=resolve_outputs,
        )

    def stacks(self, **query):
        """Return a generator of stacks

        :param kwargs query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of stack objects
        :rtype: :class:`~openstack.orchestration.v1.stack.Stack`
        """
        return self._list(_stack.Stack, **query)

    def get_stack(self, stack, resolve_outputs=True):
        """Get a single stack

        :param stack: The value can be the ID of a stack or a
            :class:`~openstack.orchestration.v1.stack.Stack` instance.
        :param resolve_outputs: Whether stack should contain outputs resolved.

        :returns: One :class:`~openstack.orchestration.v1.stack.Stack`
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when no resource can be found.
        """
        return self._get(_stack.Stack, stack, resolve_outputs=resolve_outputs)

    def update_stack(self, stack, *, preview=False, **attrs):
        """Update a stack

        :param stack: The value can be the ID of a stack or a
            :class:`~openstack.orchestration.v1.stack.Stack` instance.
        :param kwargs attrs: The attributes to update on the stack
            represented by ``value``.

        :returns: The updated stack
        :rtype: :class:`~openstack.orchestration.v1.stack.Stack`
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when no resource can be found.
        """
        res = self._get_resource(_stack.Stack, stack, **attrs)
        return res.commit(self, preview)

    def delete_stack(self, stack, ignore_missing=True):
        """Delete a stack

        :param stack: The value can be either the ID of a stack or a
            :class:`~openstack.orchestration.v1.stack.Stack`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the stack does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent stack.

        :returns: ``None``
        """
        self._delete(_stack.Stack, stack, ignore_missing=ignore_missing)

    def check_stack(self, stack):
        """Check a stack's status

        Since this is an asynchronous action, the only way to check the result
        is to track the stack's status.

        :param stack: The value can be either the ID of a stack or an instance
            of :class:`~openstack.orchestration.v1.stack.Stack`.
        :returns: ``None``
        """
        if isinstance(stack, _stack.Stack):
            stk_obj = stack
        else:
            stk_obj = _stack.Stack.existing(id=stack)

        stk_obj.check(self)

    def abandon_stack(self, stack):
        """Abandon a stack's without deleting it's resources

        :param stack: The value can be either the ID of a stack or an instance
            of :class:`~openstack.orchestration.v1.stack.Stack`.
        :returns: ``None``
        """
        res = self._get_resource(_stack.Stack, stack)
        return res.abandon(self)

    def export_stack(self, stack):
        """Get the stack data in JSON format

        :param stack: The value can be the ID or a name or
            an instance of :class:`~openstack.orchestration.v1.stack.Stack`
        :returns: A dictionary containing the stack data.
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when no resource can be found.
        """
        if isinstance(stack, _stack.Stack):
            obj = stack
        else:
            obj = self._find(_stack.Stack, stack, ignore_missing=False)
        return obj.export(self)

    def suspend_stack(self, stack):
        """Suspend a stack status

        :param stack: The value can be either the ID of a stack or an instance
            of :class:`~openstack.orchestration.v1.stack.Stack`.
        :returns: ``None``
        """
        res = self._get_resource(_stack.Stack, stack)
        res.suspend(self)

    def resume_stack(self, stack):
        """Resume a stack status

        :param stack: The value can be either the ID of a stack or an instance
            of :class:`~openstack.orchestration.v1.stack.Stack`.
        :returns: ``None``
        """
        res = self._get_resource(_stack.Stack, stack)
        res.resume(self)

    def get_stack_template(self, stack):
        """Get template used by a stack

        :param stack: The value can be the ID of a stack or an instance of
            :class:`~openstack.orchestration.v1.stack.Stack`

        :returns: One object of
            :class:`~openstack.orchestration.v1.stack_template.StackTemplate`
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when no resource can be found.
        """
        if isinstance(stack, _stack.Stack):
            obj = stack
        else:
            obj = self._find(_stack.Stack, stack, ignore_missing=False)

        return self._get(
            _stack_template.StackTemplate,
            requires_id=False,
            stack_name=obj.name,
            stack_id=obj.id,
        )

    def get_stack_environment(self, stack):
        """Get environment used by a stack

        :param stack: The value can be the ID of a stack or an instance of
            :class:`~openstack.orchestration.v1.stack.Stack`

        :returns: One object of
            :class:`~openstack.orchestration.v1.stack_environment.StackEnvironment`
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            resource can be found.
        """
        if isinstance(stack, _stack.Stack):
            obj = stack
        else:
            obj = self._find(_stack.Stack, stack, ignore_missing=False)

        return self._get(
            _stack_environment.StackEnvironment,
            requires_id=False,
            stack_name=obj.name,
            stack_id=obj.id,
        )

    def get_stack_files(self, stack):
        """Get files used by a stack

        :param stack: The value can be the ID of a stack or an instance of
            :class:`~openstack.orchestration.v1.stack.Stack`

        :returns: A dictionary containing the names and contents of all files
            used by the stack.
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when the stack cannot be found.
        """
        if isinstance(stack, _stack.Stack):
            stk = stack
        else:
            stk = self._find(_stack.Stack, stack, ignore_missing=False)

        obj = _stack_files.StackFiles(stack_name=stk.name, stack_id=stk.id)
        return obj.fetch(self)

    def resources(self, stack, **query):
        """Return a generator of resources

        :param stack: This can be a stack object, or the name of a stack
            for which the resources are to be listed.
        :param kwargs query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of resource objects if the stack exists and
            there are resources in it. If the stack cannot be found,
            an exception is thrown.
        :rtype: A generator of
            :class:`~openstack.orchestration.v1.resource.Resource`
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when the stack cannot be found.
        """
        # first try treat the value as a stack object or an ID
        if isinstance(stack, _stack.Stack):
            obj = stack
        else:
            obj = self._find(_stack.Stack, stack, ignore_missing=False)

        return self._list(
            _resource.Resource, stack_name=obj.name, stack_id=obj.id, **query
        )

    def create_software_config(self, **attrs):
        """Create a new software config from attributes

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~openstack.orchestration.v1.software_config.SoftwareConfig`,
            comprised of the properties on the SoftwareConfig class.

        :returns: The results of software config creation
        :rtype:
            :class:`~openstack.orchestration.v1.software_config.SoftwareConfig`
        """
        return self._create(_sc.SoftwareConfig, **attrs)

    def software_configs(self, **query):
        """Returns a generator of software configs

        :param dict query: Optional query parameters to be sent to limit the
            software configs returned.
        :returns: A generator of software config objects.
        :rtype:
            :class:`~openstack.orchestration.v1.software_config.SoftwareConfig`
        """
        return self._list(_sc.SoftwareConfig, **query)

    def get_software_config(self, software_config):
        """Get details about a specific software config.

        :param software_config: The value can be the ID of a software config
            or a instace of
            :class:`~openstack.orchestration.v1.software_config.SoftwareConfig`,

        :returns: An object of type
            :class:`~openstack.orchestration.v1.software_config.SoftwareConfig`
        """
        return self._get(_sc.SoftwareConfig, software_config)

    def delete_software_config(self, software_config, ignore_missing=True):
        """Delete a software config

        :param software_config: The value can be either the ID of a software
            config or an instance of
            :class:`~openstack.orchestration.v1.software_config.SoftwareConfig`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the software config does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent software config.
        :returns: ``None``
        """
        self._delete(
            _sc.SoftwareConfig, software_config, ignore_missing=ignore_missing
        )

    def create_software_deployment(self, **attrs):
        """Create a new software deployment from attributes

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~openstack.orchestration.v1.software_deployment.SoftwareDeployment`,
            comprised of the properties on the SoftwareDeployment class.

        :returns: The results of software deployment creation
        :rtype:
            :class:`~openstack.orchestration.v1.software_deployment.SoftwareDeployment`
        """
        return self._create(_sd.SoftwareDeployment, **attrs)

    def software_deployments(self, **query):
        """Returns a generator of software deployments

        :param dict query: Optional query parameters to be sent to limit the
            software deployments returned.
        :returns: A generator of software deployment objects.
        :rtype:
            :class:`~openstack.orchestration.v1.software_deployment.SoftwareDeployment`
        """
        return self._list(_sd.SoftwareDeployment, **query)

    def get_software_deployment(self, software_deployment):
        """Get details about a specific software deployment resource

        :param software_deployment: The value can be the ID of a software
            deployment or an instace of
            :class:`~openstack.orchestration.v1.software_deployment.SoftwareDeployment`,

        :returns: An object of type
            :class:`~openstack.orchestration.v1.software_deployment.SoftwareDeployment`
        """
        return self._get(_sd.SoftwareDeployment, software_deployment)

    def delete_software_deployment(
        self, software_deployment, ignore_missing=True
    ):
        """Delete a software deployment

        :param software_deployment: The value can be either the ID of a
            software deployment or an instance of
            :class:`~openstack.orchestration.v1.software_deployment.SoftwareDeployment`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the software deployment does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent software deployment.
        :returns: ``None``
        """
        self._delete(
            _sd.SoftwareDeployment,
            software_deployment,
            ignore_missing=ignore_missing,
        )

    def update_software_deployment(self, software_deployment, **attrs):
        """Update a software deployment

        :param server: Either the ID of a software deployment or an instance of
            :class:`~openstack.orchestration.v1.software_deployment.SoftwareDeployment`
        :param dict attrs: The attributes to update on the software deployment
            represented by ``software_deployment``.

        :returns: The updated software deployment
        :rtype:
            :class:`~openstack.orchestration.v1.software_deployment.SoftwareDeployment`
        """
        return self._update(
            _sd.SoftwareDeployment, software_deployment, **attrs
        )

    def validate_template(
        self, template, environment=None, template_url=None, ignore_errors=None
    ):
        """Validates a template.

        :param template: The stack template on which the validation is
            performed.
        :param environment: A JSON environment for the stack, if provided.
        :param template_url: A URI to the location containing the stack
            template for validation. This parameter is only
            required if the ``template`` parameter is None.
            This parameter is ignored if ``template`` is
            specified.
        :param ignore_errors: A string containing comma separated error codes
            to ignore. Currently the only valid error code
            is '99001'.
        :returns: The result of template validation.
        :raises: :class:`~openstack.exceptions.InvalidRequest` if neither
            `template` not `template_url` is provided.
        :raises: :class:`~openstack.exceptions.HttpException` if the template
            fails the validation.
        """
        if template is None and template_url is None:
            raise exceptions.InvalidRequest(
                "'template_url' must be specified when template is None"
            )

        tmpl = _template.Template.new()
        return tmpl.validate(
            self,
            template,
            environment=environment,
            template_url=template_url,
            ignore_errors=ignore_errors,
        )

    def get_template_contents(
        self,
        template_file=None,
        template_url=None,
        template_object=None,
        files=None,
    ):
        try:
            return template_utils.get_template_contents(
                template_file=template_file,
                template_url=template_url,
                template_object=template_object,
                files=files,
            )
        except Exception as e:
            raise exceptions.SDKException(
                f"Error in processing template files: {str(e)}"
            )

    # ========== Stack events ==========

    def stack_events(self, stack, resource_name=None, **attr):
        """Get a stack events

        :param stack: The value can be the ID of a stack or an instance of
            :class:`~openstack.orchestration.v1.stack.Stack`
        :param resource_name: The name of resource. If the resource_name is not
            None, the base_path changes.

        :returns: A generator of stack_events objects
        :rtype: :class:`~openstack.orchestration.v1.stack_event.StackEvent`
        """

        if isinstance(stack, _stack.Stack):
            obj = stack
        else:
            obj = self._get(_stack.Stack, stack)

        if resource_name:
            base_path = '/stacks/%(stack_name)s/%(stack_id)s/resources/%(resource_name)s/events'
            return self._list(
                _stack_event.StackEvent,
                stack_name=obj.name,
                stack_id=obj.id,
                resource_name=resource_name,
                base_path=base_path,
                **attr,
            )

        return self._list(
            _stack_event.StackEvent,
            stack_name=obj.name,
            stack_id=obj.id,
            **attr,
        )

    # ========== Utilities ==========

    def wait_for_status(
        self,
        res: resource.ResourceT,
        status: str,
        failures: list[str] | None = None,
        interval: int | float | None = 2,
        wait: int | None = None,
        attribute: str = 'status',
        callback: ty.Callable[[int], None] | None = None,
    ) -> resource.ResourceT:
        """Wait for the resource to be in a particular status.

        :param session: The session to use for making this request.
        :param resource: The resource to wait on to reach the status. The
            resource must have a status attribute specified via ``attribute``.
        :param status: Desired status of the resource.
        :param failures: Statuses that would indicate the transition
            failed such as 'ERROR'. Defaults to ['ERROR'].
        :param interval: Number of seconds to wait between checks.
        :param wait: Maximum number of seconds to wait for transition.
            Set to ``None`` to wait forever.
        :param attribute: Name of the resource attribute that contains the
            status.
        :param callback: A callback function. This will be called with a single
            value, progress. This is API specific but is generally a percentage
            value from 0-100.

        :return: The updated resource.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
            transition to status failed to occur in ``wait`` seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
            transitioned to one of the states in ``failures``.
        :raises: :class:`~AttributeError` if the resource does not have a
            ``status`` attribute
        """
        return resource.wait_for_status(
            self, res, status, failures, interval, wait, attribute, callback
        )

    def wait_for_delete(
        self,
        res: resource.ResourceT,
        interval: int = 2,
        wait: int = 120,
        callback: ty.Callable[[int], None] | None = None,
    ) -> resource.ResourceT:
        """Wait for a resource to be deleted.

        :param res: The resource to wait on to be deleted.
        :param interval: Number of seconds to wait before to consecutive
            checks.
        :param wait: Maximum number of seconds to wait before the change.
        :param callback: A callback function. This will be called with a single
            value, progress, which is a percentage value from 0-100.

        :returns: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
            to delete failed to occur in the specified seconds.
        """
        return resource.wait_for_delete(self, res, interval, wait, callback)

    def _get_cleanup_dependencies(self):
        return {
            'orchestration': {'before': ['compute', 'network', 'identity']}
        }

    def _service_cleanup(
        self,
        dry_run=True,
        client_status_queue=None,
        identified_resources=None,
        filters=None,
        resource_evaluation_fn=None,
        skip_resources=None,
    ):
        if self.should_skip_resource_cleanup("stack", skip_resources):
            return

        stacks = []
        for obj in self.stacks():
            need_delete = self._service_cleanup_del_res(
                self.delete_stack,
                obj,
                dry_run=dry_run,
                client_status_queue=client_status_queue,
                identified_resources=identified_resources,
                filters=filters,
                resource_evaluation_fn=resource_evaluation_fn,
            )
            if not dry_run and need_delete:
                stacks.append(obj)

        for stack in stacks:
            self.wait_for_delete(stack)



# Copyright 2017 Rackspace, US Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack.load_balancer.v2 import health_monitor
from openstack.tests.unit import base


EXAMPLE = {
    'admin_state_up': True,
    'created_at': '2017-07-17T12:14:57.233772',
    'delay': 10,
    'expected_codes': '200, 202',
    'http_method': 'HEAD',
    'id': uuid.uuid4(),
    'max_retries': 2,
    'max_retries_down': 3,
    'name': 'test_health_monitor',
    'operating_status': 'ONLINE',
    'pools': [{'id': uuid.uuid4()}],
    'pool_id': uuid.uuid4(),
    'project_id': uuid.uuid4(),
    'provisioning_status': 'ACTIVE',
    'timeout': 4,
    'type': 'HTTP',
    'updated_at': '2017-07-17T12:16:57.233772',
    'url_path': '/health_page.html',
}


class TestPoolHealthMonitor(base.TestCase):
    def test_basic(self):
        test_hm = health_monitor.HealthMonitor()
        self.assertEqual('healthmonitor', test_hm.resource_key)
        self.assertEqual('healthmonitors', test_hm.resources_key)
        self.assertEqual('/lbaas/healthmonitors', test_hm.base_path)
        self.assertTrue(test_hm.allow_create)
        self.assertTrue(test_hm.allow_fetch)
        self.assertTrue(test_hm.allow_commit)
        self.assertTrue(test_hm.allow_delete)
        self.assertTrue(test_hm.allow_list)

    def test_make_it(self):
        test_hm = health_monitor.HealthMonitor(**EXAMPLE)
        self.assertTrue(test_hm.is_admin_state_up)
        self.assertEqual(EXAMPLE['created_at'], test_hm.created_at)
        self.assertEqual(EXAMPLE['delay'], test_hm.delay)
        self.assertEqual(EXAMPLE['expected_codes'], test_hm.expected_codes)
        self.assertEqual(EXAMPLE['http_method'], test_hm.http_method)
        self.assertEqual(EXAMPLE['id'], test_hm.id)
        self.assertEqual(EXAMPLE['max_retries'], test_hm.max_retries)
        self.assertEqual(EXAMPLE['max_retries_down'], test_hm.max_retries_down)
        self.assertEqual(EXAMPLE['name'], test_hm.name)
        self.assertEqual(EXAMPLE['operating_status'], test_hm.operating_status)
        self.assertEqual(EXAMPLE['pools'], test_hm.pools)
        self.assertEqual(EXAMPLE['pool_id'], test_hm.pool_id)
        self.assertEqual(EXAMPLE['project_id'], test_hm.project_id)
        self.assertEqual(
            EXAMPLE['provisioning_status'], test_hm.provisioning_status
        )
        self.assertEqual(EXAMPLE['timeout'], test_hm.timeout)
        self.assertEqual(EXAMPLE['type'], test_hm.type)
        self.assertEqual(EXAMPLE['updated_at'], test_hm.updated_at)
        self.assertEqual(EXAMPLE['url_path'], test_hm.url_path)

        self.assertDictEqual(
            {
                'limit': 'limit',
                'marker': 'marker',
                'created_at': 'created_at',
                'updated_at': 'updated_at',
                'name': 'name',
                'project_id': 'project_id',
                'tags': 'tags',
                'any_tags': 'tags-any',
                'not_tags': 'not-tags',
                'not_any_tags': 'not-tags-any',
                'operating_status': 'operating_status',
                'provisioning_status': 'provisioning_status',
                'is_admin_state_up': 'admin_state_up',
                'delay': 'delay',
                'expected_codes': 'expected_codes',
                'http_method': 'http_method',
                'max_retries': 'max_retries',
                'max_retries_down': 'max_retries_down',
                'pool_id': 'pool_id',
                'timeout': 'timeout',
                'type': 'type',
                'url_path': 'url_path',
            },
            test_hm._query_mapping._mapping,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Find resources from the DNS service.

For a full guide see
https://docs.openstack.org/openstacksdk/latest/user/guides/dns.html
"""


def find_zone(conn, name_or_id):
    print(f"Find Zone: {name_or_id}")

    zone = conn.dns.find_zone(name_or_id)

    if zone:
        print(zone)
        return zone
    else:
        print("Zone not found.")
        return None


def find_recordset(conn, name_or_id, recordset_name, recordset_type=None):
    print(f"Find Recordset: {recordset_name} in Zone: {name_or_id}")

    zone = conn.dns.find_zone(name_or_id)

    if not zone:
        print("Zone not found.")
        return None

    zone_id = zone.id

    try:
        if recordset_type:
            recordset = conn.dns.find_recordset(
                zone_id, recordset_name, type=recordset_type
            )
        else:
            recordset = conn.dns.find_recordset(zone_id, recordset_name)

        if recordset:
            print(recordset)
            return recordset
        else:
            print("Recordset not found in Zone.")
            return None

    except Exception as e:
        print(f"{e}")
        return None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class ConsoleAuthToken(resource.Resource):
    resource_key = 'console'
    base_path = '/os-console-auth-tokens'

    # capabilities
    allow_fetch = True

    _max_microversion = '2.99'

    # Properties
    #: Instance UUID
    instance_uuid = resource.Body('instance_uuid')
    #: Hypervisor host
    host = resource.Body('host')
    #: Hypervisor port
    port = resource.Body('port')
    #: Hypervisor TLS port
    tls_port = resource.Body('tls_port')
    #: Internal access path
    internal_access_path = resource.Body('internal_access_path')



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_groups
----------------------------------

Functional tests for keystone group resource.
"""

from openstack import exceptions
from openstack.tests.functional import base


class TestGroup(base.KeystoneBaseFunctionalTest):
    def setUp(self):
        super().setUp()

        if not self.operator_cloud:
            self.skipTest("Operator cloud is required for this test")

        self.group_prefix = self.getUniqueString('group')
        self.addCleanup(self._cleanup_groups)

    def _cleanup_groups(self):
        exception_list = list()
        for group in self.operator_cloud.list_groups():
            if group['name'].startswith(self.group_prefix):
                try:
                    self.operator_cloud.delete_group(group['id'])
                except Exception as e:
                    exception_list.append(str(e))
                    continue

        if exception_list:
            # Raise an error: we must make users aware that something went
            # wrong
            raise exceptions.SDKException('\n'.join(exception_list))

    def test_create_group(self):
        group_name = self.group_prefix + '_create'
        group = self.operator_cloud.create_group(group_name, 'test group')

        for key in ('id', 'name', 'description', 'domain_id'):
            self.assertIn(key, group)
        self.assertEqual(group_name, group['name'])
        self.assertEqual('test group', group['description'])

    def test_delete_group(self):
        group_name = self.group_prefix + '_delete'

        group = self.operator_cloud.create_group(group_name, 'test group')
        self.assertIsNotNone(group)

        self.assertTrue(self.operator_cloud.delete_group(group_name))

        results = self.operator_cloud.search_groups(
            filters=dict(name=group_name)
        )
        self.assertEqual(0, len(results))

    def test_delete_group_not_exists(self):
        self.assertFalse(self.operator_cloud.delete_group('xInvalidGroupx'))

    def test_search_groups(self):
        group_name = self.group_prefix + '_search'

        # Shouldn't find any group with this name yet
        results = self.operator_cloud.search_groups(
            filters=dict(name=group_name)
        )
        self.assertEqual(0, len(results))

        # Now create a new group
        group = self.operator_cloud.create_group(group_name, 'test group')
        self.assertEqual(group_name, group['name'])

        # Now we should find only the new group
        results = self.operator_cloud.search_groups(
            filters=dict(name=group_name)
        )
        self.assertEqual(1, len(results))
        self.assertEqual(group_name, results[0]['name'])

    def test_update_group(self):
        group_name = self.group_prefix + '_update'
        group_desc = 'test group'

        group = self.operator_cloud.create_group(group_name, group_desc)
        self.assertEqual(group_name, group['name'])
        self.assertEqual(group_desc, group['description'])

        updated_group_name = group_name + '_xyz'
        updated_group_desc = group_desc + ' updated'
        updated_group = self.operator_cloud.update_group(
            group_name, name=updated_group_name, description=updated_group_desc
        )
        self.assertEqual(updated_group_name, updated_group['name'])
        self.assertEqual(updated_group_desc, updated_group['description'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import service_provider
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'service_type': 'L3_ROUTER_NAT',
    'name': '4',
    'default': False,
}


class TestServiceProvider(base.TestCase):
    def test_basic(self):
        sot = service_provider.ServiceProvider()

        self.assertEqual('service_providers', sot.resources_key)
        self.assertEqual('/service-providers', sot.base_path)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = service_provider.ServiceProvider(**EXAMPLE)
        self.assertEqual(EXAMPLE['service_type'], sot.service_type)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['default'], sot.is_default)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from examples.connect import EXAMPLE_IMAGE_NAME

"""
Create resources with the Image service.

For a full guide see
https://docs.openstack.org/openstacksdk/latest/user/guides/image.html
"""


def upload_image(conn):
    print("Upload Image:")

    # Load fake image data for the example.
    data = 'This is fake image data.'

    # Build the image attributes and upload the image.
    image_attrs = {
        'name': EXAMPLE_IMAGE_NAME,
        'data': data,
        'disk_format': 'raw',
        'container_format': 'bare',
        'visibility': 'public',
    }
    conn.image.upload_image(**image_attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_baremetal_ports
----------------------------------

Tests for baremetal port related operations
"""

from testscenarios import load_tests_apply_scenarios as load_tests  # noqa

from openstack import exceptions
from openstack.tests import fakes
from openstack.tests.unit import base


class TestBaremetalPort(base.IronicTestCase):
    def setUp(self):
        super().setUp()
        self.fake_baremetal_node = fakes.make_fake_machine(
            self.name, self.uuid
        )
        # TODO(TheJulia): Some tests below have fake ports,
        # since they are required in some processes. Lets refactor
        # them at some point to use self.fake_baremetal_port.
        self.fake_baremetal_port = fakes.make_fake_port(
            '00:01:02:03:04:05', node_id=self.uuid
        )
        self.fake_baremetal_port2 = fakes.make_fake_port(
            '0a:0b:0c:0d:0e:0f', node_id=self.uuid
        )

    def test_list_nics(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(resource='ports', append=['detail']),
                    json={
                        'ports': [
                            self.fake_baremetal_port,
                            self.fake_baremetal_port2,
                        ]
                    },
                ),
            ]
        )

        return_value = self.cloud.list_nics()
        self.assertEqual(2, len(return_value))
        self.assertSubdict(self.fake_baremetal_port, return_value[0])
        self.assert_calls()

    def test_list_nics_failure(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(resource='ports', append=['detail']),
                    status_code=400,
                )
            ]
        )
        self.assertRaises(exceptions.SDKException, self.cloud.list_nics)
        self.assert_calls()

    def test_list_nics_for_machine(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        resource='ports',
                        append=['detail'],
                        qs_elements=[
                            'node_uuid={}'.format(
                                self.fake_baremetal_node['uuid']
                            )
                        ],
                    ),
                    json={
                        'ports': [
                            self.fake_baremetal_port,
                            self.fake_baremetal_port2,
                        ]
                    },
                ),
            ]
        )

        return_value = self.cloud.list_nics_for_machine(
            self.fake_baremetal_node['uuid']
        )
        self.assertEqual(2, len(return_value))
        self.assertSubdict(self.fake_baremetal_port, return_value[0])
        self.assert_calls()

    def test_list_nics_for_machine_failure(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        resource='ports',
                        append=['detail'],
                        qs_elements=[
                            'node_uuid={}'.format(
                                self.fake_baremetal_node['uuid']
                            )
                        ],
                    ),
                    status_code=400,
                )
            ]
        )

        self.assertRaises(
            exceptions.SDKException,
            self.cloud.list_nics_for_machine,
            self.fake_baremetal_node['uuid'],
        )
        self.assert_calls()

    def test_get_nic_by_mac(self):
        mac = self.fake_baremetal_port['address']
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        resource='ports',
                        append=['detail'],
                        qs_elements=[f'address={mac}'],
                    ),
                    json={'ports': [self.fake_baremetal_port]},
                ),
            ]
        )

        return_value = self.cloud.get_nic_by_mac(mac)

        self.assertSubdict(self.fake_baremetal_port, return_value)
        self.assert_calls()

    def test_get_nic_by_mac_failure(self):
        mac = self.fake_baremetal_port['address']
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        resource='ports',
                        append=['detail'],
                        qs_elements=[f'address={mac}'],
                    ),
                    json={'ports': []},
                ),
            ]
        )

        self.assertIsNone(self.cloud.get_nic_by_mac(mac))

        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.object_store.v1 import info
from openstack.tests.unit import base


class TestInfo(base.TestCase):
    def setUp(self):
        super().setUp()

    def test_get_info_url(self):
        sot = info.Info()
        test_urls = {
            'http://object.cloud.example.com': 'http://object.cloud.example.com/info',
            'http://object.cloud.example.com/': 'http://object.cloud.example.com/info',
            'http://object.cloud.example.com/v1': 'http://object.cloud.example.com/info',
            'http://object.cloud.example.com/v1/': 'http://object.cloud.example.com/info',
            'http://object.cloud.example.com/swift': 'http://object.cloud.example.com/swift/info',
            'http://object.cloud.example.com/swift/': 'http://object.cloud.example.com/swift/info',
            'http://object.cloud.example.com/v1.0': 'http://object.cloud.example.com/info',
            'http://object.cloud.example.com/swift/v1.0': 'http://object.cloud.example.com/swift/info',
            'http://object.cloud.example.com/v111': 'http://object.cloud.example.com/info',
            'http://object.cloud.example.com/v111/test': 'http://object.cloud.example.com/info',
            'http://object.cloud.example.com/v1/test': 'http://object.cloud.example.com/info',
            'http://object.cloud.example.com/swift/v1.0/test': 'http://object.cloud.example.com/swift/info',
            'http://object.cloud.example.com/v1.0/test': 'http://object.cloud.example.com/info',
            'https://object.cloud.example.com/swift/v1/AUTH_%(tenant_id)s': 'https://object.cloud.example.com/swift/info',
            'https://object.cloud.example.com/swift/v1/AUTH_%(project_id)s': 'https://object.cloud.example.com/swift/info',
            'https://object.cloud.example.com/services/swift/v1/AUTH_%(project_id)s': 'https://object.cloud.example.com/services/swift/info',
            'https://object.cloud.example.com/services/swift/v1/AUTH_%(project_id)s/': 'https://object.cloud.example.com/services/swift/info',
            'https://object.cloud.example.com/info/v1/AUTH_%(project_id)s/': 'https://object.cloud.example.com/info/info',
        }
        for uri_k, uri_v in test_urls.items():
            self.assertEqual(sot._get_info_url(uri_k), uri_v)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import hashlib
import io

from openstack import exceptions
from openstack import utils


def _verify_checksum(md5, checksum):
    if checksum:
        digest = md5.hexdigest()
        if digest != checksum:
            raise exceptions.InvalidResponse(
                f"checksum mismatch: {checksum} != {digest}"
            )


class DownloadMixin:
    id: str
    base_path: str

    def fetch(
        self,
        session,
        requires_id=True,
        base_path=None,
        error_message=None,
        skip_cache=False,
        *,
        resource_response_key=None,
        microversion=None,
        **params,
    ): ...

    def download(
        self,
        session,
        stream=False,
        output=None,
        chunk_size=1024 * 1024,
    ):
        """Download the data contained in an image"""
        # TODO(briancurtin): This method should probably offload the get
        # operation into another thread or something of that nature.
        url = utils.urljoin(self.base_path, self.id, 'file')
        resp = session.get(url, stream=stream)

        # See the following bug report for details on why the checksum
        # code may sometimes depend on a second GET call.
        # https://storyboard.openstack.org/#!/story/1619675
        checksum = resp.headers.get("Content-MD5")

        if checksum is None:
            # If we don't receive the Content-MD5 header with the download,
            # make an additional call to get the image details and look at
            # the checksum attribute.
            details = self.fetch(session)
            checksum = details.checksum

        md5 = hashlib.md5(usedforsecurity=False)
        if output:
            try:
                if isinstance(output, io.IOBase):
                    for chunk in resp.iter_content(chunk_size=chunk_size):
                        output.write(chunk)
                        md5.update(chunk)
                else:
                    with open(output, 'wb') as fd:
                        for chunk in resp.iter_content(chunk_size=chunk_size):
                            fd.write(chunk)
                            md5.update(chunk)
                _verify_checksum(md5, checksum)

                return resp
            except Exception as e:
                raise exceptions.SDKException(f"Unable to download image: {e}")
        # if we are returning the repsonse object, ensure that it
        # has the content-md5 header so that the caller doesn't
        # need to jump through the same hoops through which we
        # just jumped.
        if stream:
            resp.headers['content-md5'] = checksum
            return resp

        if checksum is not None:
            _verify_checksum(
                hashlib.md5(resp.content, usedforsecurity=False), checksum
            )
        else:
            session.log.warning(
                "Unable to verify the integrity of image %s", (self.id)
            )

        return resp



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.database.v1 import user
from openstack.tests.unit import base


INSTANCE_ID = 'INSTANCE_ID'

CREATING = {
    'databases': '1',
    'name': '2',
    'password': '3',
}


class TestUser(base.TestCase):
    def test_basic(self):
        sot = user.User()
        self.assertEqual('user', sot.resource_key)
        self.assertEqual('users', sot.resources_key)
        self.assertEqual('/instances/%(instance_id)s/users', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make(self):
        sot = user.User(**CREATING)
        self.assertEqual(CREATING['name'], sot.id)
        self.assertEqual(CREATING['databases'], sot.databases)
        self.assertEqual(CREATING['name'], sot.name)
        self.assertEqual(CREATING['name'], sot.id)
        self.assertEqual(CREATING['password'], sot.password)

    def test_create(self):
        sot = user.User(instance_id=INSTANCE_ID, **CREATING)
        result = sot._prepare_request()
        self.assertEqual(result.body, {sot.resources_key: CREATING})



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

from openstack.block_storage.v3 import block_storage_summary as summary
from openstack.tests.unit import base


BLOCK_STORAGE_SUMMARY_312 = {
    "total_size": "4",
    "total_count": "2",
    "metadata": {"key1": "value1"},
}


BLOCK_STORAGE_SUMMARY_326 = copy.deepcopy(BLOCK_STORAGE_SUMMARY_312)
BLOCK_STORAGE_SUMMARY_326['metadata'] = {"key1": "value1"}


class TestBlockStorageSummary(base.TestCase):
    def test_basic(self):
        summary_resource = summary.BlockStorageSummary()
        self.assertEqual(None, summary_resource.resource_key)
        self.assertEqual(None, summary_resource.resources_key)
        self.assertEqual("/volumes/summary", summary_resource.base_path)
        self.assertTrue(summary_resource.allow_fetch)
        self.assertFalse(summary_resource.allow_create)
        self.assertFalse(summary_resource.allow_commit)
        self.assertFalse(summary_resource.allow_delete)
        self.assertFalse(summary_resource.allow_list)

    def test_get_summary_312(self):
        summary_resource = summary.BlockStorageSummary(
            **BLOCK_STORAGE_SUMMARY_312
        )
        self.assertEqual(
            BLOCK_STORAGE_SUMMARY_312["total_size"],
            summary_resource.total_size,
        )
        self.assertEqual(
            BLOCK_STORAGE_SUMMARY_312["total_count"],
            summary_resource.total_count,
        )

    def test_get_summary_326(self):
        summary_resource = summary.BlockStorageSummary(
            **BLOCK_STORAGE_SUMMARY_326
        )
        self.assertEqual(
            BLOCK_STORAGE_SUMMARY_326["total_size"],
            summary_resource.total_size,
        )
        self.assertEqual(
            BLOCK_STORAGE_SUMMARY_326["total_count"],
            summary_resource.total_count,
        )
        self.assertEqual(
            BLOCK_STORAGE_SUMMARY_326["metadata"], summary_resource.metadata
        )
